# 2022.2.18
import json, sys, time, fire, os, en

def readline(infile, sepa=None):
	with open(infile, 'r') as fp:
		while True:
			line = fp.readline()
			if not line: break
			yield line.strip().split(sepa) if sepa else line.strip()

class util(object):
	def __init__(self): pass 

	def es_postag(self, infile, outfile=None, snt='snt'):
		''' clec.njson -> clec.njson.postag 
		{"snt":..., } => {"snt":, ... ,"postag": }
		2022.2.18
		'''
		print("started:", infile , flush=True)
		if not outfile: outfile = infile + ".postag"
		with open(outfile, 'w') as fw : 
			for line in readline(infile): 
				try:
					arr =json.loads(line) 
					doc = en.nlp(arr.get(snt,'').strip())
					arr['postag'] = "^ " + ' '.join([f"{t.text}_{t.text.lower()}_{t.lemma_}_{t.pos_}_{t.tag_}" if t.text != t.text.lower() else f"{t.text}_{t.lemma_}_{t.pos_}_{t.tag_}" for t in doc])  + ' $'
					fw.write( json.dumps(arr) + "\n")
				except Exception as e:
					print("ex:", e, line)
		print("finished:", infile , flush=True)

	def gram(self, infile, outfile=None):
		''' gram.txt => gram.txt.postag '''
		if not hasattr(gram, 'nlp'):
			gram.nlp = spacy.load('en_core_web_sm')  # 3.1.1
			#t = nlp.get_pipe('tagger')
			#>>> nlp.pipe_names #['tok2vec', 'tagger', 'parser', 'attribute_ruler', 'lemmatizer', 'ner']
			#[nlp.remove_pipe(n) for n in ['tok2vec','parser', 'attribute_ruler','lemmatizer', 'ner'] ]
			[gram.nlp.remove_pipe(n) for n in ['parser',  'ner'] ]
		if not outfile: outfile = infile + ".postag"
		with open(outfile, 'w') as fw:
			for line in readline(infile):
				try:
					arr = line.strip().split("\t")[0:2]
					if len(arr) != 2: continue
					doc = gram.nlp(arr[0])
					poslist = " ".join([t.pos_ for t in doc])
					fw.write(f"{line.strip()}\t{poslist}\n")
				except Exception as e:
					print("ex:", e, line)
		print ("finished:", infile, outfile)

if __name__ == '__main__':
	fire.Fire(util)



word_awl ={"abandon",
"abandoned",
"abandoning",
"abandonment",
"abandons",
"abstract",
"abstraction",
"abstractions",
"abstractly",
"abstracts",
"academy",
"academia",
"academic",
"academically",
"academics",
"academies",
"access",
"accessed",
"accesses",
"accessibility",
"accessible",
"accessing",
"inaccessible",
"accommodate",
"accommodated",
"accommodates",
"accommodating",
"accommodation",
"accompany",
"accompanied",
"accompanies",
"accompaniment",
"accompanying",
"unaccompanied",
"accumulate",
"accumulated",
"accumulating",
"accumulation",
"accumulates",
"accurate",
"accuracy",
"accurately",
"inaccuracy",
"inaccuracies",
"inaccurate",
"achieve",
"achievable",
"achieved",
"achievement",
"achievements",
"achieves",
"achieving",
"acknowledge",
"acknowledged",
"acknowledges",
"acknowledging",
"acknowledgement",
"acknowledgements",
"acquire",
"acquired",
"acquires",
"acquiring",
"acquisition",
"acquisitions",
"adapt",
"adaptability",
"adaptable",
"adaptation",
"adaptations",
"adapted",
"adapting",
"adaptive",
"adapts",
"adequate",
"adequacy",
"adequately",
"inadequacies",
"inadequacy",
"inadequate",
"inadequately",
"adjacent",
"adjust",
"adjusted",
"adjusting",
"adjustment",
"adjustments",
"adjusts",
"readjust",
"readjusted",
"readjusting",
"readjustment",
"readjustments",
"readjusts",
"administrate",
"administrates",
"administration",
"administrations",
"administrative",
"administratively",
"administrator",
"administrators",
"adult",
"adulthood",
"adults",
"advocate",
"advocacy",
"advocated",
"advocates",
"advocating",
"affect",
"affected",
"affecting",
"affective",
"affects",
"unaffected",
"aggregate",
"aggregated",
"aggregates",
"aggregating",
"aggregation",
"aid",
"aided",
"aiding",
"aids",
"unaided",
"albeit",
"allocate",
"allocated",
"allocates",
"allocating",
"allocation",
"allocations",
"alter",
"alterable",
"alteration",
"alterations",
"altered",
"altering",
"alternate",
"alternating",
"alters",
"unalterable",
"unaltered",
"alternative",
"alternatively",
"alternatives",
"ambiguous",
"ambiguities",
"ambiguity",
"unambiguous",
"unambiguously",
"amend",
"amended",
"amending",
"amendment",
"amendments",
"amends",
"analogy",
"analogies",
"analogous",
"analyse",
"analysed",
"analyser",
"analysers",
"analyses",
"analysing",
"analysis",
"analyst",
"analysts",
"analytic",
"analytical",
"analytically",
"analyze",
"analyzed",
"analyzes",
"analyzing",
"annual",
"annually",
"anticipate",
"anticipated",
"anticipates",
"anticipating",
"anticipation",
"unanticipated",
"apparent",
"apparently",
"append",
"appendix",
"appended",
"appends",
"appending",
"appendices",
"appendixes",
"appreciate",
"appreciable",
"appreciably",
"appreciated",
"appreciates",
"appreciating",
"appreciation",
"unappreciated",
"approach",
"approachable",
"approached",
"approaches",
"approaching",
"unapproachable",
"appropriate",
"appropriately",
"appropriateness",
"inappropriate",
"inappropriately",
"approximate",
"approximated",
"approximately",
"approximates",
"approximating",
"approximation",
"approximations",
"arbitrary",
"arbitrariness",
"arbitrarily",
"area",
"areas",
"aspect",
"aspects",
"assemble",
"assembled",
"assembles",
"assemblies",
"assembling",
"assembly",
"assess",
"assessable",
"assessed",
"assesses",
"assessing",
"assessment",
"assessments",
"reassess",
"reassessed",
"reassessing",
"reassessment",
"assign",
"assigned",
"assigning",
"assignment",
"assignments",
"assigns",
"reassign",
"reassigned",
"reassigning",
"reassigns",
"unassigned",
"assist",
"assistance",
"assistant",
"assistants",
"assisted",
"assisting",
"assists",
"unassisted",
"assume",
"assumed",
"assumes",
"assuming",
"assumption",
"assumptions",
"assure",
"assurance",
"assurances",
"assured",
"assuredly",
"assures",
"assuring",
"attach",
"attached",
"attaches",
"attaching",
"attachment",
"attachments",
"unattached",
"attain",
"attainable",
"attained",
"attaining",
"attainment",
"attainments",
"attains",
"unattainable",
"attitude",
"attitudes",
"attribute",
"attributable",
"attributed",
"attributes",
"attributing",
"attribution",
"author",
"authored",
"authoring",
"authors",
"authorship",
"authority",
"authoritative",
"authorities",
"automate",
"automatic",
"automated",
"automates",
"automating",
"automatically",
"automation",
"available",
"availability",
"unavailable",
"aware",
"awareness",
"unaware",
"behalf",
"benefit",
"beneficial",
"beneficiary",
"beneficiaries",
"benefited",
"benefiting",
"benefits",
"bias",
"biased",
"biases",
"biasing",
"unbiased",
"bond",
"bonded",
"bonding",
"bonds",
"brief",
"brevity",
"briefed",
"briefing",
"briefly",
"briefs",
"bulk",
"bulky",
"capable",
"capabilities",
"capability",
"incapable",
"capacity",
"capacities",
"incapacitate",
"incapacitated",
"category",
"categories",
"categorisation",
"categorise",
"categorised",
"categorises",
"categorising",
"categorization",
"categorized",
"categorizes",
"categorizing",
"cease",
"ceased",
"ceaseless",
"ceases",
"ceasing",
"challenge",
"challenged",
"challenger",
"challengers",
"challenges",
"challenging",
"channel",
"channelled",
"channelling",
"channels",
"chapter",
"chapters",
"chart",
"charted",
"charting",
"charts",
"uncharted",
"chemical",
"chemically",
"chemicals",
"circumstance",
"circumstances",
"cite",
"citation",
"citations",
"cited",
"citing",
"cites",
"civil",
"clarify",
"clarification",
"clarified",
"clarifies",
"clarifying",
"clarity",
"classic",
"classical",
"classics",
"clause",
"clauses",
"code",
"coded",
"codes",
"coding",
"coherent",
"coherence",
"coherently",
"incoherent",
"incoherently",
"coincide",
"coincided",
"coincides",
"coinciding",
"coincidence",
"coincidences",
"coincident",
"coincidental",
"collapse",
"collapsed",
"collapses",
"collapsible",
"collapsing",
"colleague",
"colleagues",
"commence",
"commenced",
"commences",
"commencement",
"commencing",
"recommences",
"recommenced",
"recommencing",
"comment",
"commentaries",
"commentary",
"commentator",
"commentators",
"commented",
"commenting",
"comments",
"commission",
"commissioned",
"commissioner",
"commissioners",
"commissioning",
"commissions",
"commit",
"commitment",
"commitments",
"commits",
"committed",
"committing",
"commodity",
"commodities",
"communicate",
"communicable",
"communicated",
"communicates",
"communicating",
"communication",
"communications",
"communicative",
"uncommunicative",
"community",
"communities",
"compatible",
"compatibility",
"incompatibility",
"incompatible",
"compensate",
"compensated",
"compensates",
"compensating",
"compensation",
"compensations",
"compensatory",
"compile",
"compilation",
"compilations",
"compiled",
"compiles",
"compiling",
"complement",
"complementary",
"complemented",
"complementing",
"complements",
"complex",
"complexities",
"complexity",
"component",
"components",
"compound",
"compounded",
"compounding",
"compounds",
"comprehensive",
"comprehensively",
"comprise",
"comprised",
"comprises",
"comprising",
"compute",
"computation",
"computational",
"computations",
"computable",
"computer",
"computed",
"computerised",
"computers",
"computing",
"conceive",
"conceivable",
"conceivably",
"conceived",
"conceives",
"conceiving",
"inconceivable",
"inconceivably",
"concentrate",
"concentrated",
"concentrates",
"concentrating",
"concentration",
"concept",
"conception",
"concepts",
"conceptual",
"conceptualisation",
"conceptualise",
"conceptualised",
"conceptualises",
"conceptualising",
"conceptually",
"conclude",
"concluded",
"concludes",
"concluding",
"conclusion",
"conclusions",
"conclusive",
"conclusively",
"inconclusive",
"inconclusively",
"concurrent",
"concurrently",
"conduct",
"conducted",
"conducting",
"conducts",
"confer",
"conference",
"conferences",
"conferred",
"conferring",
"confers",
"confine",
"confined",
"confines",
"confining",
"unconfined",
"confirm",
"confirmation",
"confirmed",
"confirming",
"confirms",
"conflict",
"conflicted",
"conflicting",
"conflicts",
"conform",
"conformable",
"conformance",
"conformation",
"conformed",
"conforming",
"conformist",
"conformists",
"conformity",
"conforms",
"nonconformist",
"nonconformists",
"nonconformity",
"non-conformist",
"non-conformists",
"non-conformity",
"consent",
"consensus",
"consented",
"consenting",
"consents",
"consequent",
"consequence",
"consequences",
"consequently",
"considerable",
"considerably",
"consist",
"consisted",
"consistency",
"consistent",
"consistently",
"consisting",
"consists",
"inconsistencies",
"inconsistency",
"inconsistent",
"constant",
"constancy",
"constantly",
"constants",
"inconstancy",
"inconstantly",
"constitute",
"constituencies",
"constituency",
"constituent",
"constituents",
"constituted",
"constitutes",
"constituting",
"constitution",
"constitutions",
"constitutional",
"constitutionally",
"constitutive",
"unconstitutional",
"constrain",
"constrained",
"constraining",
"constrains",
"constraint",
"constraints",
"unconstrained",
"construct",
"constructed",
"constructing",
"construction",
"constructions",
"constructive",
"constructs",
"reconstruct",
"reconstructed",
"reconstructing",
"reconstruction",
"reconstructs",
"consult",
"consultancy",
"consultant",
"consultants",
"consultation",
"consultations",
"consultative",
"consulted",
"consults",
"consulting",
"consume",
"consumed",
"consumer",
"consumers",
"consumes",
"consuming",
"consumption",
"contact",
"contactable",
"contacted",
"contacting",
"contacts",
"contemporary",
"contemporaries",
"context",
"contexts",
"contextual",
"contextualise",
"contextualised",
"contextualising",
"contextualize",
"contextualized",
"contextualizing",
"contract",
"contracted",
"contracting",
"contractor",
"contractors",
"contracts",
"contradict",
"contradicted",
"contradicting",
"contradiction",
"contradictions",
"contradictory",
"contradicts",
"contrary",
"contrarily",
"contrast",
"contrasted",
"contrasting",
"contrastive",
"contrasts",
"contribute",
"contributed",
"contributes",
"contributing",
"contribution",
"contributions",
"contributor",
"contributors",
"controversy",
"controversies",
"controversial",
"controversially",
"uncontroversial",
"convene",
"convention",
"convenes",
"convened",
"convening",
"conventional",
"conventionally",
"conventions",
"unconventional",
"converse",
"conversely",
"convert",
"conversion",
"conversions",
"converted",
"convertible",
"converting",
"converts",
"convince",
"convinced",
"convinces",
"convincing",
"convincingly",
"unconvinced",
"cooperate",
"cooperated",
"cooperates",
"cooperating",
"cooperation",
"cooperative",
"cooperatively",
"co-operate",
"co-operated",
"co-operates",
"co-operation",
"co-operative",
"co-operatively",
"coordinate",
"coordinated",
"coordinates",
"coordinating",
"coordination",
"coordinator",
"coordinators",
"co-ordinate",
"co-ordinated",
"co-ordinates",
"co-ordinating",
"co-ordination",
"co-ordinator",
"co-ordinators",
"core",
"cores",
"coring",
"cored",
"corporate",
"corporates",
"corporation",
"corporations",
"correspond",
"corresponded",
"correspondence",
"corresponding",
"correspondingly",
"corresponds",
"couple",
"coupled",
"coupling",
"couples",
"create",
"created",
"creates",
"creating",
"creation",
"creations",
"creative",
"creatively",
"creativity",
"creator",
"creators",
"recreate",
"recreated",
"recreates",
"recreating",
"credit",
"credited",
"crediting",
"creditor",
"creditors",
"credits",
"criteria",
"criterion",
"crucial",
"crucially",
"culture",
"cultural",
"culturally",
"cultured",
"cultures",
"uncultured",
"currency",
"currencies",
"cycle",
"cycled",
"cycles",
"cyclic",
"cyclical",
"cycling",
"data",
"debate",
"debatable",
"debated",
"debates",
"debating",
"decade",
"decades",
"decline",
"declined",
"declines",
"declining",
"deduce",
"deduced",
"deduces",
"deducing",
"deduction",
"deductions",
"define",
"definable",
"defined",
"defines",
"defining",
"definition",
"definitions",
"redefine",
"redefined",
"redefines",
"redefining",
"undefined",
"definite",
"definitely",
"definitive",
"indefinite",
"indefinitely",
"demonstrate",
"demonstrable",
"demonstrably",
"demonstrated",
"demonstrates",
"demonstrating",
"demonstration",
"demonstrations",
"demonstrative",
"demonstratively",
"demonstrator",
"demonstrators",
"denote",
"denotation",
"denotations",
"denoted",
"denotes",
"denoting",
"deny",
"deniable",
"denial",
"denials",
"denied",
"denies",
"denying",
"undeniable",
"depress",
"depressed",
"depresses",
"depressing",
"depression",
"derive",
"derivation",
"derivations",
"derivative",
"derivatives",
"derived",
"derives",
"deriving",
"design",
"designed",
"designer",
"designers",
"designing",
"designs",
"despite",
"detect",
"detectable",
"detected",
"detecting",
"detection",
"detective",
"detectives",
"detector",
"detectors",
"detects",
"deviate",
"deviated",
"deviates",
"deviating",
"deviation",
"deviations",
"device",
"devices",
"devote",
"devoted",
"devotedly",
"devotes",
"devoting",
"devotion",
"devotions",
"differentiate",
"differentiated",
"differentiates",
"differentiating",
"differentiation",
"dimension",
"dimensional",
"dimensions",
"multidimensional",
"diminish",
"diminished",
"diminishes",
"diminishing",
"diminution",
"undiminished",
"discrete",
"discretely",
"discretion",
"discretionary",
"indiscretion",
"discriminate",
"discriminated",
"discriminates",
"discriminating",
"discrimination",
"displace",
"displaced",
"displacement",
"displaces",
"displacing",
"display",
"displayed",
"displaying",
"displays",
"dispose",
"disposable",
"disposal",
"disposed",
"disposes",
"disposing",
"distinct",
"distinction",
"distinctions",
"distinctive",
"distinctively",
"distinctly",
"indistinct",
"indistinctly",
"distort",
"distorted",
"distorting",
"distortion",
"distortions",
"distorts",
"distribute",
"distributed",
"distributing",
"distribution",
"distributional",
"distributions",
"distributive",
"distributor",
"distributors",
"redistribute",
"redistributed",
"redistributes",
"redistributing",
"redistribution",
"diverse",
"diversely",
"diversification",
"diversified",
"diversifies",
"diversify",
"diversifying",
"diversity",
"document",
"documentation",
"documented",
"documenting",
"documents",
"domain",
"domains",
"domestic",
"domestically",
"domesticate",
"domesticated",
"domesticating",
"domestics",
"dominate",
"dominance",
"dominant",
"dominated",
"dominates",
"dominating",
"domination",
"draft",
"drafted",
"drafting",
"drafts",
"redraft",
"redrafted",
"redrafting",
"redrafts",
"drama",
"dramas",
"dramatic",
"dramatically",
"dramatise",
"dramatised",
"dramatising",
"dramatises",
"dramatisation",
"dramatisations",
"dramatist",
"dramatists",
"dramatization",
"dramatizations",
"dramatize",
"dramatized",
"dramatizes",
"dramatizing",
"duration",
"dynamic",
"dynamically",
"dynamics",
"economy",
"economic",
"economical",
"economically",
"economics",
"economies",
"economist",
"economists",
"uneconomical",
"edit",
"edited",
"editing",
"edition",
"editions",
"editor",
"editorial",
"editorials",
"editors",
"edits",
"element",
"elements",
"eliminate",
"eliminated",
"eliminates",
"eliminating",
"elimination",
"emerge",
"emerged",
"emergence",
"emergent",
"emerges",
"emerging",
"emphasis",
"emphasise",
"emphasised",
"emphasising",
"emphasize",
"emphasized",
"emphasizes",
"emphasizing",
"emphatic",
"emphatically",
"empirical",
"empirically",
"empiricism",
"enable",
"enabled",
"enables",
"enabling",
"encounter",
"encountered",
"encountering",
"encounters",
"energy",
"energetic",
"energetically",
"energies",
"enforce",
"enforced",
"enforcement",
"enforces",
"enforcing",
"enhance",
"enhanced",
"enhancement",
"enhances",
"enhancing",
"enormous",
"enormity",
"enormously",
"ensure",
"ensured",
"ensures",
"ensuring",
"entity",
"entities",
"environment",
"environmental",
"environmentalist",
"environmentalists",
"environmentally",
"environments",
"equate",
"equated",
"equates",
"equating",
"equation",
"equations",
"equip",
"equipment",
"equipped",
"equipping",
"equips",
"equivalent",
"equivalence",
"erode",
"eroded",
"erodes",
"eroding",
"erosion",
"error",
"erroneous",
"erroneously",
"errors",
"establish",
"disestablish",
"disestablished",
"disestablishes",
"disestablishing",
"disestablishment",
"established",
"establishes",
"establishing",
"establishment",
"establishments",
"estate",
"estates",
"estimate",
"estimated",
"estimates",
"estimating",
"estimation",
"estimations",
"over-estimate",
"overestimate",
"overestimated",
"overestimates",
"overestimating",
"underestimate",
"underestimated",
"underestimates",
"underestimating",
"ethic",
"ethical",
"ethically",
"ethics",
"unethical",
"ethnic",
"ethnicity",
"evaluate",
"evaluated",
"evaluates",
"evaluating",
"evaluation",
"evaluations",
"evaluative",
"re-evaluate",
"re-evaluated",
"re-evaluates",
"re-evaluating",
"re-evaluation",
"eventual",
"eventuality",
"eventually",
"evident",
"evidenced",
"evidence",
"evidential",
"evidently",
"evolve",
"evolution",
"evolved",
"evolving",
"evolves",
"evolutionary",
"evolutionist",
"evolutionists",
"exceed",
"exceeded",
"exceeding",
"exceeds",
"exclude",
"excluded",
"excludes",
"excluding",
"exclusion",
"exclusionary",
"exclusions",
"exclusive",
"exclusively",
"exhibit",
"exhibited",
"exhibiting",
"exhibition",
"exhibitions",
"exhibits",
"expand",
"expanded",
"expanding",
"expands",
"expansion",
"expansionism",
"expansive",
"expert",
"expertise",
"expertly",
"experts",
"explicit",
"explicitly",
"exploit",
"exploitation",
"exploited",
"exploiting",
"exploits",
"export",
"exported",
"exporter",
"exporters",
"exporting",
"exports",
"expose",
"exposed",
"exposes",
"exposing",
"exposure",
"exposures",
"external",
"externalisation",
"externalise",
"externalised",
"externalises",
"externalising",
"externalization",
"externalize",
"externalized",
"externalizes",
"externalizing",
"externally",
"extract",
"extracted",
"extracting",
"extraction",
"extracts",
"facilitate",
"facilitated",
"facilitates",
"facilities",
"facilitating",
"facilitation",
"facilitator",
"facilitators",
"facility",
"factor",
"factored",
"factoring",
"factors",
"feature",
"featured",
"features",
"featuring",
"federal",
"federation",
"federations",
"fee",
"fees",
"file",
"filed",
"files",
"filing",
"final",
"finalise",
"finalised",
"finalises",
"finalising",
"finalize",
"finalized",
"finalizes",
"finalizing",
"finality",
"finally",
"finals",
"finance",
"financed",
"finances",
"financial",
"financially",
"financier",
"financiers",
"financing",
"finite",
"infinite",
"infinitely",
"flexible",
"flexibility",
"inflexible",
"inflexibility",
"fluctuate",
"fluctuated",
"fluctuates",
"fluctuating",
"fluctuation",
"fluctuations",
"focus",
"focused",
"focuses",
"focusing",
"refocus",
"refocused",
"refocuses",
"refocusing",
"format",
"formatted",
"formatting",
"formats",
"formula",
"formulae",
"formulas",
"formulate",
"formulated",
"formulating",
"formulation",
"formulations",
"reformulate",
"reformulated",
"reformulating",
"reformulation",
"reformulations",
"forthcoming",
"foundation",
"foundations",
"founded",
"founder",
"founders",
"founding",
"unfounded",
"framework",
"frameworks",
"function",
"functional",
"functionally",
"functioned",
"functioning",
"functions",
"fund",
"funded",
"funding",
"funds",
"fundamental",
"fundamentally",
"furthermore",
"gender",
"genders",
"generate",
"generated",
"generates",
"generating",
"generation",
"generations",
"globe",
"global",
"globally",
"globalisation",
"globalization",
"goal",
"goals",
"grade",
"graded",
"grades",
"grading",
"grant",
"granted",
"granting",
"grants",
"guarantee",
"guaranteed",
"guaranteeing",
"guarantees",
"guideline",
"guidelines",
"hence",
"hierarchy",
"hierarchical",
"hierarchies",
"highlight",
"highlighted",
"highlighting",
"highlights",
"hypothesis",
"hypotheses",
"hypothesise",
"hypothesised",
"hypothesises",
"hypothesising",
"hypothesize",
"hypothesized",
"hypothesizes",
"hypothesizing",
"hypothetical",
"hypothetically",
"identical",
"identically",
"identify",
"identifiable",
"identification",
"identified",
"identifies",
"identifying",
"identities",
"identity",
"unidentifiable",
"ideology",
"ideological",
"ideologically",
"ideologies",
"ignorant",
"ignorance",
"ignore",
"ignored",
"ignores",
"ignoring",
"illustrate",
"illustrated",
"illustrates",
"illustrating",
"illustration",
"illustrations",
"illustrative",
"image",
"imagery",
"images",
"immigrate",
"immigrant",
"immigrants",
"immigrated",
"immigrates",
"immigrating",
"immigration",
"impact",
"impacted",
"impacting",
"impacts",
"implement",
"implementation",
"implemented",
"implementing",
"implements",
"implicate",
"implicated",
"implicates",
"implicating",
"implication",
"implications",
"implicit",
"implicitly",
"imply",
"implied",
"implies",
"implying",
"impose",
"imposed",
"imposes",
"imposing",
"imposition",
"incentive",
"incentives",
"incidence",
"incident",
"incidentally",
"incidents",
"incline",
"inclination",
"inclinations",
"inclined",
"inclines",
"inclining",
"income",
"incomes",
"incorporate",
"incorporated",
"incorporates",
"incorporating",
"incorporation",
"index",
"indexed",
"indexes",
"indexing",
"indicate",
"indicated",
"indicates",
"indicating",
"indication",
"indications",
"indicative",
"indicator",
"indicators",
"individual",
"individualised",
"individuality",
"individualism",
"individualist",
"individualists",
"individualistic",
"individually",
"individuals",
"induce",
"induced",
"induces",
"inducing",
"induction",
"inevitable",
"inevitability",
"inevitably",
"infer",
"inference",
"inferences",
"inferred",
"inferring",
"infers",
"infrastructure",
"infrastructures",
"inherent",
"inherently",
"inhibit",
"inhibited",
"inhibiting",
"inhibition",
"inhibitions",
"inhibits",
"initial",
"initially",
"initiate",
"initiated",
"initiates",
"initiating",
"initiation",
"initiations",
"initiative",
"initiatives",
"initiator",
"initiators",
"injure",
"injured",
"injures",
"injuries",
"injuring",
"injury",
"uninjured",
"innovate",
"innovation",
"innovated",
"innovates",
"innovating",
"innovations",
"innovative",
"innovator",
"innovators",
"input",
"inputs",
"insert",
"inserted",
"inserting",
"insertion",
"inserts",
"insight",
"insightful",
"insights",
"inspect",
"inspected",
"inspecting",
"inspection",
"inspections",
"inspector",
"inspectors",
"inspects",
"instance",
"instances",
"institute",
"instituted",
"institutes",
"instituting",
"institution",
"institutional",
"institutionalise",
"institutionalised",
"institutionalises",
"institutionalising",
"institutionalized",
"institutionalizes",
"institutionalizing",
"institutionally",
"institutions",
"instruct",
"instruction",
"instructed",
"instructing",
"instructions",
"instructive",
"instructor",
"instructors",
"instructs",
"integral",
"integrate",
"integrated",
"integrates",
"integrating",
"integration",
"integrity",
"intelligence",
"intelligent",
"intelligently",
"unintelligent",
"intense",
"intensely",
"intensification",
"intensified",
"intensifies",
"intensify",
"intensifying",
"intensity",
"intensive",
"intensively",
"interact",
"interacted",
"interacting",
"interaction",
"interactions",
"interactive",
"interactively",
"interacts",
"intermediate",
"internal",
"internalise",
"internalised",
"internalises",
"internalising",
"internalize",
"internalized",
"internalizes",
"internalizing",
"internally",
"interpret",
"interpretation",
"interpretations",
"interpretative",
"interpreted",
"interpreting",
"interpretive",
"interprets",
"misinterpret",
"misinterpretation",
"misinterpretations",
"misinterpreted",
"misinterpreting",
"misinterprets",
"reinterpret",
"reinterpreted",
"reinterprets",
"reinterpreting",
"reinterpretation",
"reinterpretations",
"interval",
"intervals",
"intervene",
"intervened",
"intervenes",
"intervening",
"intervention",
"interventions",
"intrinsic",
"intrinsically",
"invest",
"invested",
"investing",
"investment",
"investments",
"investor",
"investors",
"invests",
"reinvest",
"reinvested",
"reinvesting",
"reinvestment",
"reinvests",
"investigate",
"investigated",
"investigates",
"investigating",
"investigation",
"investigations",
"investigative",
"investigator",
"investigators",
"invoke",
"invoked",
"invokes",
"invoking",
"involve",
"involved",
"involvement",
"involves",
"involving",
"uninvolved",
"isolate",
"isolated",
"isolates",
"isolating",
"isolation",
"isolationism",
"issue",
"issued",
"issues",
"issuing",
"item",
"itemisation",
"itemise",
"itemised",
"itemises",
"itemising",
"items",
"job",
"jobs",
"journal",
"journals",
"justify",
"justifiable",
"justifiably",
"justification",
"justifications",
"justified",
"justifies",
"justifying",
"unjustified",
"label",
"labeled",
"labeling",
"labelled",
"labelling",
"labels",
"labour",
"labor",
"labored",
"labors",
"laboured",
"labouring",
"labours",
"layer",
"layered",
"layering",
"layers",
"lecture",
"lectured",
"lecturer",
"lecturers",
"lectures",
"lecturing",
"legal",
"illegal",
"illegality",
"illegally",
"legality",
"legally",
"legislate",
"legislated",
"legislates",
"legislating",
"legislation",
"legislative",
"legislator",
"legislators",
"legislature",
"levy",
"levies",
"liberal",
"liberalise",
"liberalism",
"liberalisation",
"liberalised",
"liberalises",
"liberalising",
"liberalization",
"liberalize",
"liberalized",
"liberalizes",
"liberalizing",
"liberate",
"liberated",
"liberates",
"liberation",
"liberating",
"liberator",
"liberators",
"liberally",
"liberals",
"licence",
"licences",
"license",
"licensed",
"licensing",
"licenses",
"unlicensed",
"likewise",
"link",
"linkage",
"linkages",
"linked",
"linking",
"links",
"locate",
"located",
"locating",
"location",
"locations",
"relocate",
"relocated",
"relocates",
"relocating",
"relocation",
"logic",
"illogical",
"illogically",
"logical",
"logically",
"logician",
"logicians",
"maintain",
"maintained",
"maintaining",
"maintains",
"maintenance",
"major",
"majorities",
"majority",
"manipulate",
"manipulated",
"manipulates",
"manipulating",
"manipulation",
"manipulations",
"manipulative",
"manual",
"manually",
"manuals",
"margin",
"marginal",
"marginally",
"margins",
"mature",
"immature",
"immaturity",
"maturation",
"matured",
"matures",
"maturing",
"maturity",
"maximise",
"max",
"maximised",
"maximises",
"maximising",
"maximisation",
"maximize",
"maximized",
"maximizes",
"maximizing",
"maximization",
"maximum",
"mechanism",
"mechanisms",
"media",
"mediate",
"mediated",
"mediates",
"mediating",
"mediation",
"medical",
"medically",
"medium",
"mental",
"mentality",
"mentally",
"method",
"methodical",
"methodological",
"methodologies",
"methodology",
"methods",
"migrate",
"migrant",
"migrants",
"migrated",
"migrates",
"migrating",
"migration",
"migrations",
"migratory",
"military",
"minimal",
"minimalises",
"minimalist",
"minimalists",
"minimalistic",
"minimally",
"minimise",
"minimised",
"minimises",
"minimising",
"minimize",
"minimized",
"minimizes",
"minimizing",
"minimum",
"ministry",
"ministerial",
"ministries",
"minor",
"minorities",
"minority",
"minors",
"mode",
"modes",
"modify",
"modification",
"modifications",
"modified",
"modifies",
"modifying",
"unmodified",
"monitor",
"monitored",
"monitoring",
"monitors",
"unmonitored",
"motive",
"motivate",
"motivated",
"motivates",
"motivating",
"motivation",
"motivations",
"motives",
"unmotivated",
"mutual",
"mutually",
"negate",
"negative",
"negated",
"negates",
"negating",
"negatively",
"negatives",
"network",
"networked",
"networking",
"networks",
"neutral",
"neutralisation",
"neutralise",
"neutralised",
"neutralises",
"neutralising",
"neutrality",
"neutralization",
"neutralize",
"neutralized",
"neutralizes",
"neutralizing",
"nevertheless",
"nonetheless",
"norm",
"norms",
"normal",
"abnormal",
"abnormally",
"normalisation",
"normalise",
"normalised",
"normalises",
"normalising",
"normalization",
"normalize",
"normalized",
"normalizes",
"normalizing",
"normality",
"normally",
"notion",
"notions",
"notwithstanding",
"nuclear",
"objective",
"objectively",
"objectivity",
"obtain",
"obtainable",
"obtained",
"obtaining",
"obtains",
"unobtainable",
"obvious",
"obviously",
"occupy",
"occupancy",
"occupant",
"occupants",
"occupation",
"occupational",
"occupations",
"occupied",
"occupier",
"occupiers",
"occupies",
"occupying",
"occur",
"occurred",
"occurrence",
"occurrences",
"occurring",
"occurs",
"reoccur",
"reoccurred",
"reoccurring",
"reoccurs",
"odd",
"odds",
"offset",
"offsets",
"offsetting",
"ongoing",
"option",
"optional",
"options",
"orient",
"orientate",
"orientated",
"orientates",
"orientation",
"orientating",
"oriented",
"orienting",
"orients",
"reorient",
"reorientation",
"outcome",
"outcomes",
"output",
"outputs",
"overall",
"overlap",
"overlapped",
"overlapping",
"overlaps",
"overseas",
"panel",
"panelled",
"panelling",
"panels",
"paradigm",
"paradigms",
"paragraph",
"paragraphing",
"paragraphs",
"parallel",
"paralleled",
"parallelled",
"parallelling",
"parallels",
"unparalleled",
"parameter",
"parameters",
"participate",
"participant",
"participants",
"participated",
"participates",
"participating",
"participation",
"participatory",
"partner",
"partners",
"partnership",
"partnerships",
"passive",
"passively",
"passivity",
"perceive",
"perceived",
"perceives",
"perceiving",
"perception",
"perceptions",
"percent",
"percentage",
"percentages",
"period",
"periodic",
"periodical",
"periodically",
"periodicals",
"periods",
"persist",
"persisted",
"persistence",
"persistent",
"persistently",
"persisting",
"persists",
"perspective",
"perspectives",
"phase",
"phased",
"phases",
"phasing",
"phenomenon",
"phenomena",
"phenomenal",
"philosophy",
"philosopher",
"philosophers",
"philosophical",
"philosophically",
"philosophies",
"philosophise",
"philosophised",
"philosophises",
"philosophising",
"philosophize",
"philosophized",
"philosophizes",
"philosophizing",
"physical",
"physically",
"plus",
"pluses",
"policy",
"policies",
"portion",
"portions",
"pose",
"posed",
"poses",
"posing",
"positive",
"positively",
"potential",
"potentially",
"practitioner",
"practitioners",
"precede",
"preceded",
"precedence",
"precedent",
"precedes",
"preceding",
"unprecedented",
"precise",
"imprecise",
"precisely",
"precision",
"predict",
"predictability",
"predictable",
"predictably",
"predicted",
"predicting",
"prediction",
"predictions",
"predicts",
"unpredictability",
"unpredictable",
"predominant",
"predominance",
"predominantly",
"predominate",
"predominated",
"predominates",
"predominating",
"preliminary",
"preliminaries",
"presume",
"presumably",
"presumed",
"presumes",
"presuming",
"presumption",
"presumptions",
"previous",
"previously",
"primary",
"primarily",
"prime",
"primacy",
"principal",
"principally",
"principle",
"principled",
"principles",
"unprincipled",
"prior",
"priority",
"priorities",
"prioritisation",
"prioritise",
"prioritised",
"prioritises",
"prioritising",
"prioritization",
"prioritize",
"prioritized",
"prioritizes",
"prioritizing",
"proceed",
"procedural",
"procedure",
"procedures",
"proceeded",
"proceeding",
"proceedings",
"proceeds",
"process",
"processed",
"processes",
"processing",
"professional",
"professionally",
"professionals",
"professionalism",
"prohibit",
"prohibited",
"prohibiting",
"prohibition",
"prohibitions",
"prohibitive",
"prohibits",
"project",
"projected",
"projecting",
"projection",
"projections",
"projects",
"promote",
"promoted",
"promoter",
"promoters",
"promotes",
"promoting",
"promotion",
"promotions",
"proportion",
"disproportion",
"disproportionate",
"disproportionately",
"proportional",
"proportionally",
"proportionate",
"proportionately",
"proportions",
"prospect",
"prospective",
"prospects",
"protocol",
"protocols",
"psychology",
"psychological",
"psychologically",
"psychologist",
"psychologists",
"publication",
"publications",
"publish",
"published",
"publisher",
"publishers",
"publishes",
"publishing",
"unpublished",
"purchase",
"purchased",
"purchaser",
"purchasers",
"purchases",
"purchasing",
"pursue",
"pursued",
"pursues",
"pursuing",
"pursuit",
"pursuits",
"qualitative",
"qualitatively",
"quote",
"quotation",
"quotations",
"quoted",
"quotes",
"quoting",
"radical",
"radically",
"radicals",
"random",
"randomly",
"randomness",
"range",
"ranged",
"ranges",
"ranging",
"ratio",
"ratios",
"rational",
"irrational",
"rationalisation",
"rationalisations",
"rationalise",
"rationalised",
"rationalises",
"rationalising",
"rationalism",
"rationality",
"rationalization",
"rationalizations",
"rationalize",
"rationalized",
"rationalizes",
"rationally",
"react",
"reacted",
"reacts",
"reacting",
"reaction",
"reactionaries",
"reactionary",
"reactions",
"reactive",
"reactivate",
"reactivation",
"reactor",
"reactors",
"recover",
"recoverable",
"recovered",
"recovering",
"recovers",
"recovery",
"refine",
"refined",
"refinement",
"refinements",
"refines",
"refining",
"regime",
"regimes",
"region",
"regional",
"regionally",
"regions",
"register",
"registered",
"registering",
"registers",
"registration",
"regulate",
"deregulated",
"deregulates",
"deregulating",
"deregulation",
"regulated",
"regulates",
"regulating",
"regulation",
"regulations",
"regulator",
"regulators",
"regulatory",
"unregulated",
"reinforce",
"reinforced",
"reinforcement",
"reinforcements",
"reinforces",
"reinforcing",
"reject",
"rejected",
"rejecting",
"rejection",
"rejects",
"rejections",
"relax",
"relaxation",
"relaxed",
"relaxes",
"relaxing",
"release",
"released",
"releases",
"releasing",
"relevant",
"irrelevance",
"irrelevant",
"relevance",
"reluctance",
"reluctant",
"reluctantly",
"rely",
"reliability",
"reliable",
"reliably",
"reliance",
"reliant",
"relied",
"relies",
"relying",
"unreliable",
"remove",
"removable",
"removal",
"removals",
"removed",
"removes",
"removing",
"require",
"required",
"requirement",
"requirements",
"requires",
"requiring",
"research",
"researched",
"researcher",
"researchers",
"researches",
"researching",
"reside",
"resided",
"residence",
"resident",
"residential",
"residents",
"resides",
"residing",
"resolve",
"resolution",
"resolved",
"resolves",
"resolving",
"unresolved",
"resource",
"resourced",
"resourceful",
"resources",
"resourcing",
"under-resourced",
"respond",
"responded",
"respondent",
"respondents",
"responding",
"responds",
"response",
"responses",
"responsive",
"responsiveness",
"unresponsive",
"restore",
"restoration",
"restored",
"restores",
"restoring",
"restrain",
"restrained",
"restraining",
"restrains",
"restraint",
"restraints",
"unrestrained",
"restrict",
"restricted",
"restricting",
"restriction",
"restrictions",
"restrictive",
"restrictively",
"restricts",
"unrestricted",
"retain",
"retained",
"retaining",
"retainer",
"retainers",
"retains",
"retention",
"retentive",
"reveal",
"revealed",
"revealing",
"reveals",
"revelation",
"revelations",
"revenue",
"revenues",
"reverse",
"reversal",
"reversed",
"reverses",
"reversible",
"reversing",
"reversals",
"irreversible",
"revise",
"revised",
"revises",
"revising",
"revision",
"revisions",
"revolution",
"revolutionary",
"revolutionaries",
"revolutionise",
"revolutionised",
"revolutionises",
"revolutionising",
"revolutionist",
"revolutionists",
"revolutionize",
"revolutionized",
"revolutionizes",
"revolutionizing",
"revolutions",
"rigid",
"rigidities",
"rigidity",
"rigidly",
"role",
"roles",
"route",
"routed",
"routes",
"routing",
"scenario",
"scenarios",
"schedule",
"reschedule",
"rescheduled",
"reschedules",
"rescheduling",
"scheduled",
"schedules",
"scheduling",
"unscheduled",
"scheme",
"schematic",
"schematically",
"schemed",
"schemes",
"scheming",
"scope",
"section",
"sectioned",
"sectioning",
"sections",
"sector",
"sectors",
"secure",
"insecure",
"insecurities",
"insecurity",
"secured",
"securely",
"secures",
"securing",
"securities",
"security",
"seek",
"seeking",
"seeks",
"sought",
"select",
"selected",
"selecting",
"selection",
"selections",
"selective",
"selectively",
"selector",
"selectors",
"selects",
"sequence",
"sequenced",
"sequences",
"sequencing",
"sequential",
"sequentially",
"series",
"sex",
"sexes",
"sexism",
"sexual",
"sexuality",
"sexually",
"shift",
"shifted",
"shifting",
"shifts",
"significant",
"insignificant",
"insignificantly",
"significance",
"significantly",
"signified",
"signifies",
"signify",
"signifying",
"similar",
"dissimilar",
"similarities",
"similarity",
"similarly",
"simulate",
"simulated",
"simulates",
"simulating",
"simulation",
"site",
"sites",
"so-called",
"sole",
"solely",
"somewhat",
"source",
"sourced",
"sources",
"sourcing",
"specific",
"specifically",
"specification",
"specifications",
"specificity",
"specifics",
"specify",
"specifiable",
"specified",
"specifies",
"specifying",
"unspecified",
"sphere",
"spheres",
"spherical",
"spherically",
"stable",
"instability",
"stabilisation",
"stabilise",
"stabilised",
"stabilises",
"stabilising",
"stabilization",
"stabilize",
"stabilized",
"stabilizes",
"stabilizing",
"stability",
"unstable",
"statistic",
"statistician",
"statisticians",
"statistical",
"statistically",
"statistics",
"status",
"straightforward",
"strategy",
"strategic",
"strategies",
"strategically",
"strategist",
"strategists",
"stress",
"stressed",
"stresses",
"stressful",
"stressing",
"unstressed",
"structure",
"restructure",
"restructured",
"restructures",
"restructuring",
"structural",
"structurally",
"structured",
"structures",
"structuring",
"unstructured",
"style",
"styled",
"styles",
"styling",
"stylish",
"stylise",
"stylised",
"stylises",
"stylising",
"stylize",
"stylized",
"stylizes",
"stylizing",
"submit",
"submission",
"submissions",
"submits",
"submitted",
"submitting",
"subordinate",
"subordinates",
"subordination",
"subsequent",
"subsequently",
"subsidy",
"subsidiary",
"subsidies",
"subsidise",
"subsidised",
"subsidises",
"subsidising",
"subsidize",
"subsidized",
"subsidizes",
"subsidizing",
"substitute",
"substituted",
"substitutes",
"substituting",
"substitution",
"successor",
"succession",
"successions",
"successive",
"successively",
"successors",
"sufficient",
"sufficiency",
"insufficient",
"insufficiently",
"sufficiently",
"sum",
"summation",
"summed",
"summing",
"sums",
"summary",
"summaries",
"summarise",
"summarised",
"summarises",
"summarising",
"summarize",
"summarized",
"summarizes",
"summarizing",
"supplement",
"supplementary",
"supplemented",
"supplementing",
"supplements",
"survey",
"surveyed",
"surveying",
"surveys",
"survive",
"survival",
"survived",
"survives",
"surviving",
"survivor",
"survivors",
"suspend",
"suspended",
"suspending",
"suspends",
"suspension",
"sustain",
"sustainable",
"sustainability",
"sustained",
"sustaining",
"sustains",
"sustenance",
"unsustainable",
"symbol",
"symbolic",
"symbolically",
"symbolise",
"symbolises",
"symbolised",
"symbolising",
"symbolism",
"symbolize",
"symbolized",
"symbolizes",
"symbolizing",
"symbols",
"tape",
"taped",
"tapes",
"taping",
"target",
"targeted",
"targeting",
"targets",
"task",
"tasks",
"team",
"teamed",
"teaming",
"teams",
"technical",
"technically",
"technique",
"techniques",
"technology",
"technological",
"technologically",
"temporary",
"temporarily",
"tense",
"tension",
"tensely",
"tenser",
"tensest",
"tensions",
"terminate",
"terminal",
"terminals",
"terminated",
"terminates",
"terminating",
"termination",
"terminations",
"text",
"texts",
"textual",
"theme",
"themes",
"thematic",
"thematically",
"theory",
"theoretical",
"theoretically",
"theories",
"theorist",
"theorists",
"thereby",
"thesis",
"theses",
"topic",
"topical",
"topics",
"trace",
"traceable",
"traced",
"traces",
"tracing",
"tradition",
"non-traditional",
"traditional",
"traditionalist",
"traditionally",
"traditions",
"transfer",
"transferable",
"transference",
"transferred",
"transferring",
"transfers",
"transform",
"transformation",
"transformations",
"transformed",
"transforming",
"transforms",
"transit",
"transited",
"transiting",
"transition",
"transitional",
"transitions",
"transitory",
"transits",
"transmit",
"transmission",
"transmissions",
"transmitted",
"transmitting",
"transmits",
"transport",
"transportation",
"transported",
"transporter",
"transporters",
"transporting",
"transports",
"trend",
"trends",
"trigger",
"triggered",
"triggering",
"triggers",
"ultimate",
"ultimately",
"undergo",
"undergoes",
"undergoing",
"undergone",
"underwent",
"underlie",
"underlay",
"underlies",
"underlying",
"undertake",
"undertaken",
"undertakes",
"undertaking",
"undertook",
"uniform",
"uniformity",
"uniformly",
"unify",
"unification",
"unified",
"unifies",
"unifying",
"unique",
"uniquely",
"uniqueness",
"utilise",
"utilisation",
"utilised",
"utilises",
"utilising",
"utility",
"utilities",
"utilization",
"utilize",
"utilized",
"utilizes",
"utilizing",
"valid",
"invalidate",
"invalidity",
"validate",
"validated",
"validating",
"validation",
"validity",
"validly",
"vary",
"invariable",
"invariably",
"variability",
"variable",
"variables",
"variably",
"variance",
"variant",
"variants",
"variation",
"variations",
"varied",
"varies",
"varying",
"vehicle",
"vehicles",
"version",
"versions",
"via",
"violate",
"violated",
"violates",
"violating",
"violation",
"violations",
"virtual",
"virtually",
"visible",
"visibility",
"visibly",
"invisible",
"invisibility",
"vision",
"visions",
"visual",
"visualise",
"visualised",
"visualising",
"visualisation",
"visualize",
"visualized",
"visualizing",
"visualization",
"visually",
"volume",
"volumes",
"vol",
"voluntary",
"voluntarily",
"volunteer",
"volunteering",
"volunteered",
"volunteers",
"welfare",
"whereas",
"whereby",
"widespread",
"affectively",
"appropriacy",
"inappropriacy",
"unassessed",
"communicatively",
"componentry",
"conformability",
"uncontextualised",
"uncontextualized",
"indiscrete",
"exclusionist",
"externality",
"focussed",
"focussing",
"refocussed",
"refocusses",
"refocussing",
"funder",
"funders",
"intenseness",
"intension",
"liberations",
"maturational",
"minimalisation",
"minimalise",
"minimalised",
"minimalising",
"minimalization",
"minimalize",
"minimalized",
"minimalizes",
"minimalizing",
"deregister",
"deregistered",
"deregistering",
"deregisters",
"deregistration",
"unresourceful",
"unrestrictive",
"summarisation",
"summarisations",
"summarization",
"summarizations",
"utiliser",
"utilisers"}

is_word_awl = lambda w : w in word_awl

def submit(port, host='127.0.0.1',  db=0):
	import redis
	r = redis.Redis(host=host, port=port, db=db)
	for w in word_awl: 
		r.sadd("dict:awl", w )
		r.hset('dict:word_level', w, 'awl')
	print(">> finished loading awl")

if __name__ == '__main__':
	submit(6311)



#!/usr/bin/env python -W ignore::DeprecationWarning 
# 2022.2.17, upgrade of sntbs, spacy 3.1.1-based, one single file
import sqlite3,json,collections, fire, traceback,sys
from collections import	UserDict,Counter,defaultdict
from tqdm import tqdm

from en import * #import en # need 3.1.1
from en import terms,verbnet
attach = lambda doc: ( terms.attach(doc), verbnet.attach(doc), doc.user_data )[-1]  # return ssv, defaultdict(dict)

class Spacybs(UserDict): # change to a neutral name, such as Dbdict ?  then Docbs(Dbdict),   add compress later ? 2021-2-3
	def	__init__(self, filename, tablename='spacybs'): 
		self.filename =	filename
		self.tablename = tablename
		self.conn =	sqlite3.connect(self.filename, check_same_thread=False) 
		self.conn.execute(f'CREATE TABLE IF NOT EXISTS {self.tablename} (key varchar(512) PRIMARY KEY, value blob)')
		self.conn.execute('PRAGMA synchronous=OFF')
		self.conn.commit()

	def	__str__(self): 	return "SqliteDict(%s)"	% (self.filename)
	def	__repr__(self): return str(self)  #	no need	of something complex
	def	__len__(self):	return self.conn.execute('SELECT COUNT(*) FROM	"%s"' %	self.tablename).fetchone()[0]
	def	count(self):	return self.conn.execute('SELECT count(*) FROM "%s"'% self.tablename).fetchone()[0]

	def	keys(self, start=0, len=-1):  
		for key in self.conn.execute(f'SELECT key FROM {self.tablename} ORDER BY rowid limit {start},{len}' ).fetchall(): yield key[0]
	def	values(self, start=0, len=-1): 
		for	value in self.conn.execute(f'SELECT value FROM {self.tablename} ORDER BY rowid  limit {start},{len}').fetchall(): yield value[0]
	def	items(self, start=0, len=-1): 
		for rowid, key, value in self.conn.execute(f'SELECT rowid, key, value FROM	{self.tablename} ORDER BY rowid limit {start},{len}' ).fetchall(): 	yield rowid, key, value
	def	docs(self, start=0, len=-1): 
		for	value in self.conn.execute(f'SELECT value FROM {self.tablename} ORDER BY rowid  limit {start},{len}').fetchall(): yield from_docbin(value[0])

	def	__contains__(self, key): return self.conn.execute('SELECT 1 FROM "%s" WHERE key = ?' %	self.tablename, (key,)).fetchone() is not None

	def	__getitem__(self, key):
		item = self.conn.execute(f'SELECT value FROM "{self.tablename}" WHERE key = ? limit 1', (key,)).fetchone()
		return None if item	is None else item[0] # else json.loads(...)
	def get(self, key): return self[key]

	def	__setitem__(self, key, value): 	self.conn.execute('REPLACE	INTO "%s" (key,	value) VALUES (?,?)' % self.tablename,	(key, value))
	def set(self, key, value): self[key] = value
	def	__delitem__(self, key): self.conn.execute('DELETE FROM	"%s" WHERE key = ?'	% self.tablename,	(key,))
	def	__iter__(self): 		return self.keys()
	def	close(self): 	self.conn.commit()

def init_db(filename):
	conn = sqlite3.connect(filename, check_same_thread=False)  
	conn.execute(f'DROP TABLE IF EXISTS fts')
	conn.execute('''CREATE VIRTUAL TABLE if not exists fts USING fts5(snt, terms, columnsize=0, detail=full,tokenize = "unicode61 remove_diacritics 0 tokenchars '-_'")''') #self.conn.execute('''CREATE VIRTUAL TABLE if not exists fts USING fts5(snt, terms, columnsize=0, detail=none,tokenize = "unicode61 remove_diacritics 0 tokenchars '-_'")''')
	conn.execute('PRAGMA synchronous=OFF')
	conn.commit()
	return conn

def submit_si(dbfile, si, cutoff=0):
	conn = sqlite3.connect(dbfile, check_same_thread=False)  
	conn.execute(f'DROP TABLE IF EXISTS si')
	conn.execute(f'CREATE TABLE IF NOT EXISTS si (s varchar(128) PRIMARY KEY, i int) without rowid')
	for k,v in si.items():
		if v > cutoff : 
			conn.execute(f"replace into si(s,i) values(?,?)", (k,v))
	conn.commit()

def submit_ssi(dbfile, ssi):
	conn = sqlite3.connect(dbfile, check_same_thread=False)  
	conn.execute(f'DROP TABLE IF EXISTS ssi')
	conn.execute(f'CREATE TABLE IF NOT EXISTS ssi (s varchar(128) PRIMARY KEY, i int) without rowid')
	for k,v in ssi.items(): #redis.dm.zadd(f"{corpus}:{k}", dict(v))
		for s,i in v.items(): 
			conn.execute(f"replace into ssi(s,i) values(?,?)", (f"{k}:{s}",i))
	conn.commit()

incr	= lambda si, *names, delta = 1: [si.update({name: delta}) for name in names ] #incr(si, "one", "two")
doc_fts = lambda conn, rowid, snt, doc: conn.execute(f"insert or ignore into fts(rowid, snt,terms) values(?,?,?)", (rowid, snt, " ".join([ f"{t.pos_}_{t.lemma_}" for t in doc] + [ f"{t.tag_}_{t.lemma_}" for t in doc] + [ f"{t.dep_}_{t.head.pos_}_{t.pos_}_{t.head.lemma_}_{t.lemma_}" for t in doc if t.pos_ not in ('PRON','PUNCT') and t.dep_ in ('dobj','nsubj','advmod','acomp','amod','compound','xcomp','ccomp')]) ))

class util(object):
	
	def fts(self,dbfile, outfile):  
		''' clec.spacybs -> add fts, select * from fts where match(kps) against ('+open_VERB' in boolean mode), 2022.2.26 '''
		conn = init_db(outfile)
		for rowid, snt, bs in tqdm(Spacybs(dbfile).items()) :
			try:
				doc = frombs(bs)
				kps = " ".join([ f"{t.pos_}_{t.lemma_}" for t in doc] + [ f"{t.tag_}_{t.lemma_}" for t in doc] + [ f"{t.dep_}_{t.head.pos_}_{t.pos_}_{t.head.lemma_}_{t.lemma_}" for t in doc if t.pos_ not in ('PRON','PUNCT') and t.dep_ in ('dobj','nsubj','advmod','acomp','amod','compound','xcomp','ccomp')])
				conn.execute(f"insert or ignore into fts(rowid, snt,terms) values(?,?,?)", (rowid, snt, kps ))
			except Exception as e:
				print ("ex:", e, rowid, snt)
		conn.commit()
		print (f"[fts] finished, {dbfile}")

	def index(self,dbfile, outfile=None, fts=True):  
		''' clec.spacybs -> clec.sqlite, 2022.2.27 '''
		if not outfile : outfile  = dbfile.split('.')[0] + ".sqlite" 
		print (f"[index] started, {dbfile} -> {outfile}", flush=True)
		conn = init_db(outfile)
		ssi = defaultdict(Counter) #si = Counter()
		for rowid, snt, bs in tqdm(Spacybs(dbfile).items()) :
			try:
				doc = frombs(bs)
				terms.zset_ssi(doc, ssi)
				if fts: doc_fts(conn, rowid, snt, doc) 
			except Exception as e:
				print ("ex:", e, rowid, snt)
		conn.commit()
		print (f"start to dump ssi, {dbfile} -> {outfile}")
		submit_ssi(outfile, ssi)
		print (f"[index] finished, {dbfile} -> {outfile}")

	def si(self,dbfile, outfile):  
		''' clec.spacybs -> add si, 2022.2.26 '''
		si = Counter()
		for rowid, snt, bs in tqdm(Spacybs(dbfile).items()) :
			try:
				doc = frombs(bs)
				for t in doc:
					si.update({f"{t.pos_}:{t.lemma_}":1}) # VERB:book 
					si.update({f"LEM:{t.lemma_}":1})
					si.update({f"LOW:{t.text.lower()}":1})
					si.update({f"{t.lemma_}:{t.pos_}:{t.tag_}":1}) #book:VERB:VBD
					si.update({f"{t.lemma_}:LEX:{t.text.lower()}":1}) #book:LEX:booked 
					if t.pos_ not in ("PROPN","PUNCT"): si.update({f"{t.lemma_}:POS:{t.pos_}":1})
					si.update({f"{t.dep_}_{t.head.lemma_}_{t.lemma_}:{t.head.lemma_} {t.lemma_}":1})
			except Exception as e:
				print ("ex:", e, rowid, snt)
		submit_si(outfile, si)
		print (f"[si] finished, {dbfile}")

	def train(self, sntfile, dbfile=None): 
		''' train clec.snt => clec.sntbs, 2021.8.1 '''
		if not dbfile : dbfile = sntfile.split(".")[0].lower() + ".spacybs"
		print("started:", sntfile, dbfile,flush=True)
		db = Spacybs(dbfile)
		for line in tqdm(open(sntfile,'r').readlines()):
			try:
				snt = line.strip()
				if snt and not snt in db: 
					db[snt] = tobs(nlp(snt))
			except Exception as e:
				print ("parse ex:", e, line)
		db.close()
		print("finished:", sntfile, dbfile)

	def topika(self,dbfile, name=None, host='127.0.0.1', port=9311, db_bs=0, db_dm=5):  
		''' clec.spacybs -> :  snt:clec(list) , {snt}:bs(hash),  2022.1.20 '''
		import redis
		rdm = redis.Redis(host=host, port=port, db=db_dm, decode_responses=True)
		rbs = redis.Redis(host=host, port=port, db=db_bs)
		if not name: name = dbfile.split('.')[0].lower()
		rdm.delete(f"snts:{name}")
		for rowid, snt, bs in tqdm(Spacybs(dbfile).items()) :
			try:
				doc = frombs(bs)
				rdm.rpush(f"snts:{name}", snt ) 
				rbs.setnx(snt, tobs(doc))
			except Exception as e:
				print ("ex:", e, rowid, snt)
		print (f"[tobs] finished, {dbfile}, {name}")

	def idsource(self,dbfile, outfile=None ):  
		''' clec.spacybs -> clec.idsource , 2022.2.11
			{"_id": "140948871-9", "_source": {"rid": "10", "uid": "25110374", "sc": 14, "md5": "da891a7d81f7a5e43b571168cc483b6c dba0b4c99ef37cadfc4bacd61fcefa5b d6b199bfae35246564c598ac78d84c91 38a945eeff5b5a587a26dcc6560e0061 58605af6b50b01f15c0cc3ee2aa75e33 c30566c355ae09ea68673e2940d49d0a 7972c906aef51380310363093e141ef8 dabdd545400415da6d29125bf872"}}
			Then submit:  es.py idsource  clec.idsource
		'''
		if not outfile: outfile = dbfile.split(".")[0] + ".idsource"
		with open(outfile, 'w') as fw:
			for rowid, snt, bs in tqdm(Spacybs(dbfile).items()) :
				try:
					doc = frombs(bs) 
					ssv = attach(doc) 
					for id, source in ssv.items():
						source.update({"src": rowid})
						fw.write(json.dumps({"_id": f"{rowid}-{id}", "_source":source}) + "\n") 
				except Exception as ex:
					print(">>idsource ex:", ex,"\t|", rowid, snt)
		print("submit idsource finished:", dbfile, outfile)

	def toes(self,dbfile, idxname=None, eshost='127.0.0.1', esport=9200, batch=100000 , refresh=True):  
		''' clec.spacybs -> es/clec directly, 2022.2.15
			{"_id": "140948871-9", "_source": {"rid": "10", "uid": "25110374", "sc": 14, "md5": "da891a7d81f7a5e43b571168cc483b6c dba0b4c99ef37cadfc4bacd61fcefa5b d6b199bfae35246564c598ac78d84c91 38a945eeff5b5a587a26dcc6560e0061 58605af6b50b01f15c0cc3ee2aa75e33 c30566c355ae09ea68673e2940d49d0a 7972c906aef51380310363093e141ef8 dabdd545400415da6d29125bf872"}}
		'''
		from elasticsearch import Elasticsearch,helpers
		from so import config
		es	= Elasticsearch([ f"http://{eshost}:{esport}" ])  
		if not idxname : idxname = dbfile.split('.')[0].lower()
		if refresh and es.indices.exists(idxname): es.indices.delete(idxname)
		if not es.indices.exists(idxname): es.indices.create(idxname, config) 
		print ("toes started:", dbfile, idxname, es, flush=True)
		actions=[]
		for rowid, snt, bs in tqdm(Spacybs(dbfile).items()) :
			try:
				doc = frombs(bs) 
				actions.append({'_op_type':'index', '_index':idxname, '_id': rowid, '_source':{'type':'snt', 'snt':snt,
				'postag':'^ ' + ' '.join([f"{t.text}_{t.lemma_}_{t.pos_}_{t.tag_}" if t.text == t.text.lower() else f"{t.text}_{t.text.lower()}_{t.lemma_}_{t.pos_}_{t.tag_}" for t in doc]) + ' $',
				'src': rowid,  'tc': len(doc)}}) #'sid': rowid,
				ssv = attach(doc) 
				for id, source in ssv.items():
					source.update({"src":rowid}) # sid doesnot work  "sid": rowid, 
					actions.append({'_op_type':'index', '_index':idxname, '_id': f"{rowid}-{id}", '_source':source})
				if len(actions) > batch : 
					helpers.bulk(client=es,actions=actions, raise_on_error=False)
					actions = []
					print (rowid, snt , flush=True) 
			except Exception as ex:
				print(">>toes ex:", ex,"\t|", rowid, snt)
				exc_type, exc_value, exc_traceback_obj = sys.exc_info()
				traceback.print_tb(exc_traceback_obj)

		helpers.bulk(client=es,actions=actions, raise_on_error=False)
		print("toes finished:", dbfile, idxname)

	def tosnt(self,dbfile):  
		''' dump snt to console '''
		for snt in Spacybs(dbfile).keys() :
			print(snt)
			
if __name__	== '__main__':
	fire.Fire(util)

'''
def es_source(name, snt, doc, skip_punct:bool=True): 
	id  = hashlib.md5(snt.strip().encode("utf-8")).hexdigest()
	sntlen = len(doc)
	arr = {f"{name}-{id}": {"snt": snt, "type":"snt", "tc": sntlen, "awl":  sum([ len(t.text) for t in doc])/sntlen, 'postag': ' '.join(['^'] + [f"{t.text}_{t.lemma_}_{t.tag_}_{t.pos_}" for t in doc] + ['$']) } }
	
	[ arr.update({ f"{name}-{id}-trp-{t.i}" : {"src":f"{name}-{id}",'type':'trp','gov': t.head.lemma_, 'rel': f"{t.dep_}_{t.head.pos_}_{t.pos_}", 'dep': t.lemma_ }}) for t in doc if not skip_punct or t.dep_ not in ('punct')]
	[ arr.update({ f"{name}-{id}-tok-{t.i}" : {'type':'tok', 'src': f"{name}-{id}", 'lex': t.text, 'low': t.text.lower(), 'lem': t.lemma_, 'pos': t.pos_, 'tag': t.tag_, 'i':t.i, 'head': t.head.i }}) for t in doc]
	[ arr.update({ f"{name}-{id}-np-{np.start}" : {'type':'np', 'src': f"{name}-{id}", 'lem': doc[np.end-1].lemma_, 'chunk': np.text, }}) for np in doc.noun_chunks]
	return arr 

		conn.execute(f'DROP TABLE IF EXISTS termsnt')
		conn.execute(f'CREATE TABLE IF NOT EXISTS termsnt (term varchar(128) PRIMARY KEY, snt text)')
					termsnt[f"{t.pos_}:{t.lemma_}"] = snt  # highlight later 
					termsnt[f"{t.dep_}_{t.head.lemma_}_{t.lemma_}:{t.head.lemma_} {t.lemma_}"] = snt 
		for term,snt in termsnt.items():
			conn.execute(f"insert or ignore into termsnt(term,snt) values(?,?)", (term, snt))
		conn.commit()

				incr(si, "sum:snt") #si.update({"sntnum":1})
				for t in doc:
					incr(si, "sum:lex",f"{t.pos_}:{t.lemma_}",f"LEM:{t.lemma_}",f"LOW:{t.text.lower()}"
					,f"{t.lemma_}:{t.pos_}:{t.tag_}" #book:VERB:VBD
					,f"{t.lemma_}:LEX:{t.text.lower()}" #book:LEX:booked 
					,f"{t.dep_}_{t.head.lemma_}_{t.lemma_}:{t.head.lemma_} {t.lemma_}")
					if t.pos_ not in ("PROPN","PUNCT"): si.update({f"{t.lemma_}:POS:{t.pos_}":1})


http://sentbase.com/lemma/lex/?q=belong

sqlite> select * from ssi where s like 'book/LEX:%';
book/LEX:book|51
book/LEX:books|40
book/LEX:booked|1
sqlite> select * from ssi where s like 'book/POS:%';
book/POS:NOUN|91
book/POS:VERB|1
sqlite> select * from ssi where s like 'dobj_VERB_NOUN:open %';
dobj_VERB_NOUN:open calendar|1
dobj_VERB_NOUN:open window|2

sqlite> select * from ssi where s like 'consider/VERB:%';
consider/VERB:VB|5
consider/VERB:#|24
consider/VERB:nsubj|12

select * from fts where snt match('booked')  limit 3;

select * from fts where terms match('dobj_VERB_NOUN_open_door')  limit 3;
select * from fts where terms match('VERB_open')  limit 3;

http://sentbase.com/trp/

sqlite> select * from ssi where s in ('dobj_VERB_NOUN:overcome difficulty','~amod_NOUN_ADJ:overcome difficulty');
dobj_VERB_NOUN:overcome difficulty|2

sqlite> select * from ssi where s like 'dobj_VERB_NOUN:overcome %' order by i desc;
dobj_VERB_NOUN:overcome difficulty|2
dobj_VERB_NOUN:overcome illness|1
dobj_VERB_NOUN:overcome dizziness|1
dobj_VERB_NOUN:overcome obstacle|1
dobj_VERB_NOUN:overcome sorrow|1
dobj_VERB_NOUN:overcome date|1

sqlite> select * from ssi where s like '~dobj_VERB_NOUN:difficulty %' order by i desc;
~dobj_VERB_NOUN:difficulty have|6
~dobj_VERB_NOUN:difficulty overcome|2
~dobj_VERB_NOUN:difficulty avoid|1
~dobj_VERB_NOUN:difficulty find|1

select snt from fts where terms match('dobj_VERB_NOUN_overcome_difficulty')  limit 10;


select * from ssi where s in ('dobj_VERB_NOUN:overcome difficulty','~amod_NOUN_ADJ:front door')



'''


# 2022.3.2, my_doc, tag_doc,  sqlite-based , | uvicorn mytag-sqlite:app --host 0.0.0.0 --port 13306 --reload 
import json,os,re, requests,traceback,fire, time,math,collections , hashlib,sys
from collections import	defaultdict, Counter

from uvirun import * 
app.title = "My Tag api, sqlite-corpus"
app.tm = "2022.3.2"

from en import * # need 3.1.1  FileNotFoundError: [Errno 2] No such file or directory: 'C:\\Users\\zhang/.cache/lmdb-spacy311'
from en.spacybs import *
from en import terms,verbnet
attach = lambda doc: ( terms.attach(doc), verbnet.attach(doc), doc.user_data )[-1]  # return ssv, defaultdict(dict)
simple_sent		= lambda doc: len([t for t in doc if t.pos_ == 'VERB' and t.dep_ != 'ROOT']) <= 0 
complex_sent	= lambda doc: len([t for t in doc if t.pos_ == 'VERB' and t.dep_ != 'ROOT']) > 0
compound_sent	= lambda doc: len([t for t in doc if t.dep_ == 'conj' and t.head.dep_ == 'ROOT']) > 0  # S.conj 

db = Spacybs("mytag.sqlite") # support es ops # db.close()

@app.get("/mytag/newdoc")
def add_text(text:str="The quick fox jumped over the lazy dog. The justice delayed is justice denied.", prefix:str='my', title:str=None, note:str="", addtag:bool=False):
	''' prefix: my/tag '''
	cursor=db.conn.cursor()
	docsnts = snts(text)
	if not title: title = docsnts[0]
	cursor.execute(f'INSERT INTO {prefix}_doc(title,body,note) VALUES (?,?,?)',(title, text, note))
	id = cursor.lastrowid
	print(id)
	for i, snt in enumerate(docsnts):
		doc = nlp(snt)
		db[snt] = tobs(doc)
		cursor.execute(f'INSERT or ignore INTO {prefix}_snt(sid, did, snt) VALUES (?,?,?)',(f"{id}-{i}",id, snt))
		#insert tok, trp, chk 
		for t in doc: 
			cursor.execute(f'INSERT or ignore INTO {prefix}_tok(sid, i, text, lem, pos, tag, dep, head) VALUES (?,?,?,?,?,?,?,?)',(f"{id}-{i}", t.i,
				t.text, t.lemma_, t.pos_, t.tag_, t.dep_, t.head.i))
			cursor.execute(f'INSERT or ignore INTO {prefix}_trp(sid, i, rel, govpos, govlem, deppos, deplem) VALUES (?,?,?,?,?,?,?)',(f"{id}-{i}", t.i,
				t.dep_, t.head.pos_, t.head.lemma_, t.pos_, t.lemma_))
		for np in doc.noun_chunks: 
			cursor.execute(f'INSERT or ignore INTO {prefix}_chk(sid, start, end, chunk, type, lem) VALUES (?,?,?,?,?,?)',(f"{id}-{i}", np.start, np.end, np.text, "np", doc[np.end-1].lemma_))
			
		if addtag: 
			stype = "简单句" if len([t for t in doc if t.pos_ == 'VERB' and t.dep_ != 'ROOT']) <= 0 else "复杂句"
			cursor.execute(f'INSERT or ignore INTO tag_stype(sid, stype) VALUES (?,?)',(f"{id}-{i}", stype))
			for np in doc.noun_chunks: 
				cursor.execute(f'INSERT or ignore INTO tag_span(sid, ibeg, iend, tag,note) VALUES (?,?,?,?,?)',(f"{id}-{i}", 
				doc[np.start].idx, doc[np.start].idx + len(np.text), "名词短语", np.text))

	db.conn.commit()
	return id #add_text(prefix="tag", addtag=True)

@app.get("/mytag/sql")
def corpus_sql(query:str="select * from tag_cate limit 2"):
	return [row for row in db.conn.execute(query).fetchall()]

def init(prefix="my"):
	try:
		db.conn.execute(f"CREATE TABLE IF NOT EXISTS {prefix}_doc (did INTEGER PRIMARY KEY AUTOINCREMENT, title varchar(256), body mediumtext not null, note text not null default '', tm timestamp )")
		db.conn.execute(f"CREATE TABLE IF NOT EXISTS {prefix}_snt ( sid varchar(32) PRIMARY KEY, did integer, snt text )")
		db.conn.execute(f'''CREATE VIRTUAL TABLE if not exists {prefix}_fts USING fts5(sid, snt, terms, columnsize=0, detail=full,tokenize = "unicode61 remove_diacritics 0 tokenchars '-_'")''') #self.conn.execute('''CREATE VIRTUAL TABLE if not exists fts USING fts5(snt, terms, columnsize=0, detail=none,tokenize = "unicode61 remove_diacritics 0 tokenchars '-_'")''')
		db.conn.execute(f'CREATE TABLE IF NOT EXISTS {prefix}_tok (sid varchar(16), i int, text varchar(64), lem varchar(64), pos varchar(16), tag varchar(16), dep varchar(16), head int)') 
		db.conn.execute(f'CREATE TABLE IF NOT EXISTS {prefix}_trp (sid varchar(16), i int, rel varchar(32), govlem varchar(64), govpos varchar(16), deplem varchar(64), deppos varchar(16))') 
		db.conn.execute(f'CREATE TABLE IF NOT EXISTS {prefix}_chk (sid varchar(16), start int not null, end int not null, chunk varchar(128) , type varchar(32), lem varchar(64))') 
		db.conn.commit()
		db.conn.execute(f"CREATE UNIQUE INDEX {prefix}_trpidx ON {prefix}_trp (sid,i)") #CREATE UNIQUE INDEX index_name on table_name (column_name);
		db.conn.execute(f"CREATE UNIQUE INDEX {prefix}_chkidx ON {prefix}_chk (sid, start)")
		db.conn.execute(f"CREATE UNIQUE INDEX {prefix}_tokidx ON {prefix}_tok (sid,i)")
		db.conn.commit()
	except Exception as ex:
		print(">>toes ex:", ex)

@app.get("/mytag/refresh_db")
def mytag_refresh_db(): 
	global db 
	db.conn.commit()
	db.conn.close()
	if os.path.exists ("mytag.sqlite") : 
		os.remove("mytag.sqlite")
	db = Spacybs("mytag.sqlite")
	db.conn.execute(f"CREATE TABLE IF NOT EXISTS tag_stype ( sid varchar(32),  stype varchar(32), note varhcar(256), tm timestamp)")
	db.conn.execute(f"CREATE TABLE IF NOT EXISTS tag_span ( sid varchar(32), ibeg int not null, iend int not null, tag varchar(128) not null, note varhcar(256) not null default '', tm timestamp)")
	db.conn.execute(f"CREATE TABLE IF NOT EXISTS tag_cate ( cate varchar(32), tag varchar(32) not null, note varhcar(256) not null default '')")
	db.conn.commit()
	db.conn.execute(f"CREATE UNIQUE INDEX tag_stypeidx ON tag_stype (sid,stype)")
	db.conn.execute(f"CREATE UNIQUE INDEX tag_spanidx ON tag_span (sid,ibeg, iend)")
	db.conn.execute(f"CREATE UNIQUE INDEX tag_tagidx ON tag_cate (cate, tag)")
	[ db.conn.execute(f"insert or ignore into tag_cate ( cate,tag ) values ('从句','{v}')") for v in ("主语从句","宾语从句")]
	db.conn.commit()
	[ init(prefix) for prefix in ('my','tag')] 
	return db 

@app.get('/sent/terms')
def snt_terms(snt:str="I think that I am going to go swimming."):  
	''' return term for stats, 2022.2.26 ''' 
	try:
		doc = nlp(snt) #doc.user_data["snt"] = snt
		doc.user_data["tok"] = [ {"i": t.i, "text":t.text, "lower":t.text.lower(), "lem":t.lemma_, "pos":t.pos_, "tag":t.tag_, "dep":t.dep_, "head":t.head.i} for t in doc]
		attach(doc) 
		return doc.user_data
	except Exception as ex:
		return  {"ex":str(ex), "snt":snt }

if __name__ == "__main__":  
	uvicorn.run(app, host='0.0.0.0', port=13306)

'''
def add_si(s,i):
	db.conn.execute(f"INSERT INTO ssi (s, i) VALUES (?, {i}) ON CONFLICT(s) DO UPDATE SET i = i + {i}", (i,))

def remove_si(s,i):
	db.conn.execute(f"update ssi set i = i - {i} where s = ?", (s,))

conn.execute(f'CREATE TABLE IF NOT EXISTS {self.tablename} (key varchar(512) PRIMARY KEY, value blob)')
conn.execute('REPLACE	INTO spacybs (key,	value) VALUES (?,?)' % self.tablename,	(key, value))
conn.execute(f'CREATE TABLE IF NOT EXISTS ssi (s varchar(128) PRIMARY KEY, i int)')
stype ( ID INTEGER PRIMARY KEY   AUTOINCREMENT, sid integer, tag varchar(100), comment varhcar(256)  
spantag ( ID INTEGER PRIMARY KEY   AUTOINCREMENT, sid integer,  ibeg int, iend int , tag varchar(128), comment varhcar(256) )

INSERT INTO ssi (s, i)
VALUES ('127.0.0.1', 1)
ON CONFLICT(s) DO UPDATE SET i = i + 1;

connection=sqlite3.connect(':memory:')
cursor=connection.cursor()
cursor.execute("CREATE TABLE foo (id integer primary key autoincrement ,
                                    username varchar(50),
                                    password varchar(50))")
cursor.execute('INSERT INTO foo (username,password) VALUES (?,?)',
               ('test','test'))
print(cursor.lastrowid)
'''


#from uvirun import * 
import json,os,uvicorn,time
from fastapi import FastAPI
from starlette.responses import JSONResponse 
from starlette.routing import Route

async def homepage(request):
    return JSONResponse({"index":"HOme"}) 

async def about(request):
    return JSONResponse({"index":"about"}) 

routes = [
    Route("/", endpoint=homepage,methods=["GET"]),
    Route("/about", endpoint=about,methods=["POST"]),
]

app=FastAPI(routes=routes)

if __name__ == '__main__':
	uvicorn.run(app, host='0.0.0.0', port=80)



# 2022.3.4  /home/ubuntu/.local/lib/python3.8/site-packages/uvirun/nlp.py
import json,fire,os
from uvirun import * 
from en import nlp,snts

@app.get("/nlp/terms")
def nlp_terms(text:str="The quick fox jumped over the lazy dog. The justice delayed is justice denied."):
	''' for sqlite indexing, 2022.3.4 '''
	tdoc = nlp(text)
	arr = []
	for sent in tdoc.sents: 
		doc = sent.as_doc()
		arr.append( { "snt": sent.text, 
		"tokens": [ {"id": t.i,"offset":t.idx, "word": t.text, "lemma":t.lemma_, "is_stop":t.is_stop, "parent": -1, "np_root": False, "pos": t.pos_, "tag": t.tag_, "dep": t.dep_,"text_with_ws": t.text_with_ws, "head": t.head.i , "sent_start": t.is_sent_start, "sent_end":t.is_sent_end}  for t in doc], 
		"triples": [ {"id":t.i,"offset":t.idx, "rel": t.dep_, "govlem":t.head.lemma_, "govpos": t.head.pos_, "deplem": t.lemma_, "deppos": t.pos_} for t in doc], 
		"chunk": [ {"id": np.start, "offset": doc[np.start].idx, "lem": doc[np.end-1].lemma_, "chunk":np.text, "end":np.end} for np in doc.noun_chunks], 
		} )
	return arr 

@app.get("/nlp/sntbr")
def nlp_sntbr(text:str="The quick fox jumped over the lazy dog. The justice delayed is justice denied.", trim:bool=True):
	'''  '''
	return snts(text, trim) 

def run(port):
	uvicorn.run(app, host='0.0.0.0', port=port)

if __name__ == "__main__":  
	fire.Fire(run)
	#uvicorn.run(app, host='0.0.0.0', port=80)


# https://github.com/yahoo/redislite

ifnone		= lambda v, default: v if v else default 
zset		= lambda key, topk = -1, asdic =False: r.zrevrange(key,0,topk,True) if not asdic else dict(r.zrevrange(key,0,topk,True))
zperc		= lambda key: util.L1_norm(r.zrevrange(key,0,-1,True)) # zsetperc("gzjc:lempos:book") #[('NOUN', 0.9894), ('VERB', 0.0106)]
zset_if		= lambda key, substr=':VB': [(k, score) for k, score in r.zrevrange(key,0,-1,True) if substr in k ] # lemtag
zdic		= lambda key,cutoff=0: { k:v for k,v in r.zrevrange(key,0,-1,True) if v > cutoff }

import math 
def likelihood(a,b,c,d, minus=None):  #from: http://ucrel.lancs.ac.uk/llwizard.html
	try:
		if a is None or a <= 0 : a = 0.000001
		if b is None or b <= 0 : b = 0.000001
		E1 = c * (a + b) / (c + d)
		E2 = d * (a + b) / (c + d)
		G2 = round(2 * ((a * math.log(a / E1)) + (b * math.log(b / E2))), 2)
		if minus or  (minus is None and a/c < b/d): G2 = 0 - G2
		return G2
	except Exception as e:
		print ("likelihood ex:",e, a,b,c,d)
		return 0
			
if __name__	== '__main__': 
	print (likelihood(1,2,3,4))


# 2022.1.22
import streamlit as st
import requests
import pandas as pd

import redis,spacy,json
from collections import defaultdict, Counter
redis.dsk	= redis.Redis("dsk.wrask.com", port=9221, db=0, decode_responses=True) 
redis.mkf	= redis.Redis("dsk.wrask.com", port=9221, db=1, decode_responses=True) 
redis.dm	= redis.Redis("dsk.wrask.com", port=9221, db=2, decode_responses=True) 
redis.bs	= redis.Redis("dsk.wrask.com", port=9221, db=3, decode_responses=False)
r = redis.dsk
if not hasattr(spacy, 'nlp'): 
	spacy.nlp		= spacy.load('en_core_web_sm')
	spacy.frombs	= lambda bs: list(spacy.tokens.DocBin().from_bytes(bs).get_docs(spacy.nlp.vocab))[0] if bs else None
	spacy.tobs		= lambda doc: ( doc_bin:= spacy.tokens.DocBin(), doc_bin.add(doc), doc_bin.to_bytes())[-1]
	spacy.getdoc	= lambda snt: ( bs := redis.bs.get(snt), doc := spacy.frombs(bs) if bs else spacy.nlp(snt), redis.bs.setnx(snt, spacy.tobs(doc)) if not bs else None )[1]

eidv_list   = lambda rid: [f"{k}-{v}" for k,v in redis.dsk.hgetall(f"rid:{rid}").items()]
rid_snts	= lambda rid: (	snts := [], [ snts.extend(json.loads(redis.dsk.hget(eidv, 'snts'))) for eidv in eidv_list(rid) ] )[0]
rid_mkfs	= lambda rid: [	json.loads(mkf) for mkf in redis.mkf.mget( rid_snts(rid)) ]
eidv_score  = lambda eidv: json.loads(redis.dsk.hget(eidv,'dsk')).get('info',{}).get('final_score',0.0)
eidv_docs  = lambda eidv: [ spacy.getdoc(snt) for snt in json.loads(redis.dsk.hget(eidv,'snts'))]
doc_term   = lambda doc, ssi:  [ ssi[t.pos_].update({t.lemma_:1}) for t in doc]
eidv_term  = lambda eidv: ( ssi := defaultdict(Counter), [ doc_term(doc,ssi)  for doc in eidv_docs(eidv)] )[0]


from math import log as ln
def likelihood(a,b,c,d, minus=None):  #from: http://ucrel.lancs.ac.uk/llwizard.html
	try:
		if a is None or a <= 0 : a = 0.000001
		if b is None or b <= 0 : b = 0.000001
		E1 = c * (a + b) / (c + d)
		E2 = d * (a + b) / (c + d)
		G2 = round(2 * ((a * ln(a / E1)) + (b * ln(b / E2))), 2)
		if minus or  (minus is None and a/c < b/d): G2 = 0 - G2
		return G2
	except Exception as e:
		print ("likelihood ex:",e, a,b,c,d)
		return 0



# 2022.2.9, http://verbs.colorado.edu/verb-index/vn/consider-29.9.php#consider-29.9-1-1-1
import spacy, traceback, sys
from spacy.matcher import Matcher
if not hasattr(spacy,'nlp'): spacy.nlp	= spacy.load('en_core_web_sm') #if not 'nlp' in dir(): nlp	= spacy.load('en_core_web_sm')  

matcher	 = Matcher(spacy.nlp.vocab)  # :1 , verb's offset 
NP_start = {"ENT_TYPE": "NP", "IS_SENT_START": True}
VERB	 = {"POS": {"IN": ["VERB"]}}
NOUN	 = {"POS": {"IN": ["NOUN","PRON","PROPN"]}}
PUNCT	 = {"IS_PUNCT": True}

matcher.add("NP V:1", [[NP_start,VERB, PUNCT]])
matcher.add("NP of NP V:3", [[ NP_start,{"LEMMA": "of"}, {"ENT_TYPE": "NP"}, VERB,PUNCT]], greedy ='LONGEST')

matcher.add("NP V NP:1", [[NOUN,VERB, NOUN,{"POS": {"IN": ["PUNCT"]}}]], greedy ='LONGEST')
matcher.add("NP V NP ADJ:1", [[NOUN,VERB, NOUN,{"POS": {"IN": ["ADJ"]}}]], greedy ='LONGEST')
matcher.add("NP V NP NP:1", [[NOUN,VERB, NOUN,NOUN]], greedy ='LONGEST')
matcher.add("NP V NP-Dative NP:1", [[NOUN,VERB, {"DEP": {"IN": ["dative"]}},NOUN]], greedy ='LONGEST')

matcher.add("NP V NP PP:1", [[NOUN,VERB, NOUN,{"DEP": {"IN": ["prep"]}}]], greedy ='LONGEST')
matcher.add("NP V NP PP PP:1", [[NOUN,VERB, NOUN,{"DEP": {"IN": ["prep"]}}, NOUN,{"DEP": {"IN": ["prep"]}}, NOUN]], greedy ='LONGEST')

matcher.add("NP V S_ING:1", [[NOUN,VERB, {"TAG": {"IN": ["VBG"]}}]], greedy ='LONGEST')
matcher.add("NP V whether/how S_INF:1", [[NOUN,VERB, {"LEMMA": {"IN": ["whether","how"]}}, {"LEMMA": {"IN": ["to"]}}, VERB]], greedy ='LONGEST')
matcher.add("NP V NP to be NP:1", [[NOUN,VERB, {"LEMMA": {"IN": ["to"]}}, {"LEMMA": {"IN": ["be"]}}, NOUN]], greedy ='LONGEST')
matcher.add("NP V that/how S:1", [[NOUN,VERB, {"LEMMA": {"IN": ["that","how"]}, "OP":"*"}, NOUN, {"POS": {"IN": ["AUX","PART"]}, "OP":"*"},{"DEP": {"IN": ["ccomp"]}}]], greedy ='LONGEST') #They considered that he was the professor.
matcher.add("NP V whether/if S:1", [[NOUN,VERB, {"LEMMA": {"IN": ["whether","if"]}}, NOUN,{"POS": {"IN": ["AUX","PART"]}, "OP":"*"}, {"DEP": {"IN": ["ccomp"]}}]], greedy ='LONGEST') #He considered whether he should come.
matcher.add("NP V what S:1", [[NOUN,VERB, {"LEMMA": {"IN": ["what"]}}, NOUN,{"POS": {"IN": ["AUX","PART"]}, "OP":"*"}, {"DEP": {"IN": ["ccomp"]}}]], greedy ='LONGEST') 
matcher.add("NP V what S_INF:1", [[NOUN,VERB, {"LEMMA": {"IN": ["what"]}}, {"LEMMA": {"IN": ["to"]}},VERB]], greedy ='LONGEST') 

def merge_np(doc):
	with doc.retokenize() as retokenizer:
		for np in doc.noun_chunks:
			attrs = {"tag": np.root.tag, "dep": np.root.dep, "ent_type": "NP"}
			retokenizer.merge(np, attrs=attrs) 
	return doc

def merge_n_of_n(doc):
	if not hasattr(merge_n_of_n, 'matcher'):
		merge_n_of_n.matcher = Matcher(spacy.nlp.vocab)
		merge_n_of_n.matcher.add("n-of-n", [[{"ENT_TYPE": "NP"}, {"LEMMA":"of"},{"ENT_TYPE": "NP"}], [{"ENT_TYPE": "NP"}, {"LEMMA":"of"},{"POS": "NOUN"}]], greedy ='LONGEST')
	with doc.retokenize() as retokenizer:
		for name, start, end in merge_n_of_n.matcher(doc):
			if end - start > 1: 
				try:
					i = doc[start].head.i
					attrs = {"pos": doc[i].pos, "tag": doc[i].tag, "dep": doc[i].dep, "lemma":doc[i].lemma, "ent_type": "NP"}
					retokenizer.merge(doc[start : end], attrs=attrs)
				except Exception as e:
					print ( "merge_n_of_n ex:", e , start, end)
	return doc

def merge_vp(doc):
	if not hasattr(merge_vp, 'matcher'):
		merge_vp.matcher = Matcher(spacy.nlp.vocab)
		merge_vp.matcher.add("vp", [[{"POS": {"IN":["AUX","PART"]}, "op": "*"}, {"POS":"VERB"},{"POS": "ADV", "op": "*"}]], greedy ='LONGEST')
	with doc.retokenize() as retokenizer:
		for name, start, end in merge_vp.matcher(doc):
			if end - start > 1: 
				try:
					i = doc[start].head.i
					attrs = {"pos": doc[i].pos, "tag": doc[i].tag, "dep": doc[i].dep, "lemma":doc[i].lemma, "ent_type": "VP"}
					retokenizer.merge(doc[start : end], attrs=attrs)
				except Exception as e:
					print ( "merge_vp ex:", e , start, end)
	return doc

def merge_pp(doc): 
	if not hasattr(merge_pp, 'matcher'):
		merge_pp.matcher = Matcher(spacy.nlp.vocab)
		merge_pp.matcher.add("pp", [[{"POS": {"IN":["ADP"]}, "op": "+"}, {"ENT_TYPE":"NP", "op": "+"}]], greedy ='LONGEST')
	with doc.retokenize() as retokenizer:
		for name, start, end in merge_pp.matcher(doc):
			if end - start > 1: 
				try:
					i = doc[start].head.i
					attrs = {"pos": doc[i].pos, "tag": doc[i].tag, "dep": doc[i].dep, "lemma":doc[i].lemma, "ent_type": "PP"}
					retokenizer.merge(doc[start : end], attrs=attrs)
				except Exception as e:
					print ( "merge_pp ex:", e , start, end)
	return doc

def merge_clause(doc): # subtree of a verb is the clause , https://subscription.packtpub.com/book/data/9781838987312/2/ch02lvl1sec13/splitting-sentences-into-clauses
	with doc.retokenize() as retokenizer:
		for v in [t for t in doc if t.pos_ == 'VERB' and t.dep_ != 'ROOT' ] : # non-root
			try:
				children = list(v.subtree)
				start = children[0].i  	
				end = children[-1].i 
				attrs = {"pos": v.pos, "tag": v.tag, "dep": v.dep, "lemma":v.lemma, "ent_type": "S." + v.dep_ } # S.advcl ,  S.conj 
				retokenizer.merge(doc[start : end+1], attrs=attrs)
			except Exception as e:
				print ( "merge_clause ex:", e, v )
	return doc

def verbnet(doc):
	merge_np(doc)
	merge_vp(doc)
	for name, ibeg, iend in matcher(doc):
		arr = spacy.nlp.vocab[name].text.split(':') 
		i = ibeg + int(arr[-1]) 
		doc.user_data[f"verbnet-{i}"] = {"type":"verbnet", "start": ibeg, "end": iend, "lem":doc[i].lemma_,"chunk":arr[0].strip()}

simple_sent		= lambda doc: len([t for t in doc if t.pos_ == 'VERB' and t.dep_ != 'ROOT']) <= 0 
complex_sent	= lambda doc: len([t for t in doc if t.pos_ == 'VERB' and t.dep_ != 'ROOT']) > 0
compound_sent	= lambda doc: len([t for t in doc if t.dep_ == 'conj' and t.head.dep_ == 'ROOT']) > 0  # S.conj 

def stype(doc): 
	doc.user_data["stype"] = {"type": "simple_sent" if len([t for t in doc if t.pos_ == 'VERB' and t.dep_ != 'ROOT']) <= 0 else "complex_sent" }
	if len([t for t in doc if t.dep_ == 'conj' and t.head.dep_ == 'ROOT']) > 0  : # S.conj 
		doc.user_data["compound_sent"] = {"type": "compound_sent" }

skeleton = lambda doc:  " ".join([ t.ent_type_ if t.ent_type_ else t.text if t.is_punct or t.dep_ == 'ROOT' else t.pos_ for t in doc ])
def ske(doc): 
	merge_clause(doc)
	doc.user_data["ske"] = {"type":"ske","lem": ','.join([t.lemma_ for t in doc if t.dep_ == 'ROOT']),"chunk": skeleton(doc)}

def clause(doc):  # {'S.prep-0': {'type': 'S.prep', 'start': 0, 'end': 2, 'lem': 'consider', 'chunk': 'Considering the possibility'}, 'S.conj-9': {'type': 'S.conj', 'start': 9, 'end': 12, 'lem': 'be', 'chunk': 'she is ok .'}}
	for v in [t for t in doc if t.pos_ == 'VERB' and t.dep_ != 'ROOT' ] : # non-root
		children = list(v.subtree)
		start = children[0].i  	#end = children[-1].i 
		type = "S." + v.dep_   # S.advcl ,  S.conj 
		doc.user_data[f"{type}-{start}"] = {"type":type, "start": start, "end":children[-1].i + 1, "lem":v.lemma_,"chunk": " ".join([c.text for c in v.subtree])}

def attach(doc):
	verbnet(doc)
	stype(doc)
	clause(doc) 
	#ske(doc)

if __name__ == "__main__":  
	doc = spacy.nlp("Considering the possibility, it was ok, and she is ok.")
	clause(doc) 
	verbnet(doc)
	stype(doc)
	ske(doc)
	print (doc.user_data)

'''
	ssv = defaultdict(dict) 
	add_clause(101, doc, ssv)
	print (ssv)
	merge_np(doc)
	merge_clause(doc)
	print(doc[0].text)
	print ( skeleton(doc))

from collections import	defaultdict
def doc_to_verbnet(doc):
	merge_np(doc)
	merge_vp(doc)
	for name, ibeg, iend in matcher(doc):
		arr = spacy.nlp.vocab[name].text.split(':') 
		offset = int(arr[-1]) 
		yield (ibeg + offset, doc[ibeg+offset].lemma_, arr[0].strip() )

def submit_verbnet(sid, doc, ssv): 	#ssv = defaultdict(dict)
	for ibeg, lem, chunk in doc_to_verbnet(doc): 
		ssv[f"{sid}-verbnet-{ibeg}"].update ({"type":"verbnet","lem":lem,"chunk":chunk})

'''


#
import json, fire, sqlite3

def dump(dbfile, outfile):
	''' '''
	print(">> started:", dbfile, flush=True)
	conn = sqlite3.connect(dbfile)
	with open(outfile, 'w') as fw : 
		for row in conn.execute(f"select * from sv" ):
			try:
				arr = {"_id": f"{row[0]}", "_source": {"v": f"{row[1]}"}}
				fw.write(json.dumps(arr) + "\n") 
			except Exception as e:
				print("ex:", e, row[0])
	print(">> finished:", dbfile)

if __name__ == '__main__': 
	fire.Fire(dump) 

'''
PUT grampos
{
 "mappings": {
   "properties": { 
     "v":{"type": "keyword", "index":false }
	 }
 }
}
'''


# 2022.3.3  for yulk.sentbase.com  | uvicorn yulk:app --host 0.0.0.0 --port 18001 --reload
import json,requests,hashlib,os
from uvirun import * 
from en import nlp, snts

from collections import	defaultdict, Counter
from dic.bnc_wordlist import * 
@app.get('/yulk/keyness')  #def bnc_keyness(si:dict, si_sum:int=None):
def text_keyness(text="I think that I am going to go to the cinema. The quick fox jumped over the lazy dog.", topk:int=20):  
	''' 单词超用显著性 ''' 
	doc = nlp(text) 
	si = Counter()
	[ si.update({t.text.lower():1}) for t in doc if not t.pos_ in ('PROPN','PUNCT') and not t.is_stop]
	res = bnc_keyness(dict(si), len(doc))
	res.sort(key=lambda x:x[-1], reverse=True)
	return res[0:topk]

from dic.word_idf import word_idf
wordidf = lambda txt:  (si := Counter(), [ si.update({word_idf[t.text.lower()]: 1}) for t in nlp(txt) if t.text.lower() in word_idf] )[0]
@app.get('/yulk/vs/wordidf') 
def vs_wordidf(text0="I think that I am going to go to the cinema. The quick fox jumped over the lazy dog.", text1="The reality that has blocked my path to become the typical successful student is that engineering and the liberal arts simply don't mix as easily as I assured in hight school.",):  
	''' 单词难度对比 ''' 
	si0 = wordidf(text0)
	si1 = wordidf(text1)
	return (si0, si1)

@app.post('/yulk/wordmap') 
def wordmap(snts:list=["I think that I am going to go to the cinema.","The quick fox jumped over the lazy dog."], poslist:str="VERB,NOUN,ADV,ADJ"):  
	''' 语义邻居 ''' 
	ssi = defaultdict(Counter)
	pos = poslist.strip().split(',')
	for snt in snts:
		doc = nlp(snt) 
		[ ssi[t.pos_].update({t.lemma_:1})  for t in doc if t.pos_ in pos]
	return ssi 

from en.spacybs import Spacybs
db = Spacybs("essaytag.sqlite") # support es ops # db.close()
db.conn.execute(f"CREATE TABLE IF NOT EXISTS doc (did INTEGER PRIMARY KEY AUTOINCREMENT, title varchar(256), body mediumtext not null, note text not null default '', tm timestamp )")
db.conn.execute(f"CREATE TABLE IF NOT EXISTS snt ( sid varchar(32) PRIMARY KEY, snt text )") # 1001-1, 1001-2, ...
db.conn.execute(f'''CREATE VIRTUAL TABLE if not exists fts USING fts5(sid, snt, terms, columnsize=0, detail=full,tokenize = "unicode61 remove_diacritics 0 tokenchars '-_'")''') #self.conn.execute('''CREATE VIRTUAL TABLE if not exists fts USING fts5(snt, terms, columnsize=0, detail=none,tokenize = "unicode61 remove_diacritics 0 tokenchars '-_'")''')
db.conn.execute(f'CREATE TABLE IF NOT EXISTS tok (sid varchar(16), i int, text varchar(64), lem varchar(64), pos varchar(16), tag varchar(16), dep varchar(16), head int)') 
db.conn.execute(f'CREATE TABLE IF NOT EXISTS trp (sid varchar(16), i int, rel varchar(32), govlem varchar(64), govpos varchar(16), deplem varchar(64), deppos varchar(16))') 
db.conn.execute(f'CREATE TABLE IF NOT EXISTS chk (sid varchar(16), start int not null, end int not null, chunk varchar(128) , type varchar(32), lem varchar(64))') 
db.conn.execute(f"CREATE UNIQUE INDEX if not exists trpidx ON trp (sid,i)") #CREATE UNIQUE INDEX index_name on table_name (column_name);
db.conn.execute(f"CREATE UNIQUE INDEX if not exists chkidx ON chk (sid, start)")
db.conn.execute(f"CREATE UNIQUE INDEX if not exists tokidx ON tok (sid,i)")
db.conn.execute(f"CREATE TABLE IF NOT EXISTS tag_stype ( sid varchar(32),  stype varchar(32), note varhcar(256), tm timestamp)")
db.conn.execute(f"CREATE TABLE IF NOT EXISTS tag_span ( sid varchar(32), ibeg int not null, iend int not null, tag varchar(128) not null, note varhcar(256) not null default '', tm timestamp)")
db.conn.execute(f"CREATE TABLE IF NOT EXISTS tag_cate ( cate varchar(32), tag varchar(32) not null, note varhcar(256) not null default '')")
db.conn.execute(f"CREATE UNIQUE INDEX if not exists tag_stypeidx ON tag_stype (sid,stype)")
db.conn.execute(f"CREATE UNIQUE INDEX if not exists tag_spanidx ON tag_span (sid,ibeg, iend)")
db.conn.execute(f"CREATE UNIQUE INDEX if not exists tag_tagidx ON tag_cate (cate, tag)")
db.conn.commit()

@app.get("/yulk/essaytag/clear")
def essaytag_clear():
	''' '''
	for name in ("doc","snt","tok","trp","chk", "fts", "tag_stype","tag_span", "tag_cate"):
		db.conn.execute(f"drop table if exists {name}")
	return db.conn.commit()

@app.get("/yulk/essaytag/newdoc")
def add_text(text:str="The quick fox jumped over the lazy dog. The justice delayed is justice denied.", title:str=None, note:str=""):
	''' prefix: my/tag '''
	cursor=db.conn.cursor()
	docsnts = snts(text,False)
	if not title: title = docsnts[0]
	cursor.execute(f'INSERT INTO doc(title,body,note) VALUES (?,?,?)',(title, text, note))
	id = cursor.lastrowid
	for i, snt in enumerate(docsnts):
		doc = nlp(snt)
		db[snt] = tobs(doc)
		cursor.execute(f'INSERT or ignore INTO snt(sid, snt) VALUES (?,?)',(f"{id}-{i}",snt))
		for t in doc: 
			cursor.execute(f'INSERT or ignore INTO tok(sid, i, text, lem, pos, tag, dep, head) VALUES (?,?,?,?,?,?,?,?)',(f"{id}-{i}", t.i,	t.text, t.lemma_, t.pos_, t.tag_, t.dep_, t.head.i))
			cursor.execute(f'INSERT or ignore INTO trp(sid, i, rel, govpos, govlem, deppos, deplem) VALUES (?,?,?,?,?,?,?)',(f"{id}-{i}", t.i,	t.dep_, t.head.pos_, t.head.lemma_, t.pos_, t.lemma_))
		for np in doc.noun_chunks: 
			cursor.execute(f'INSERT or ignore INTO chk(sid, start, end, chunk, type, lem) VALUES (?,?,?,?,?,?)',(f"{id}-{i}", np.start, np.end, np.text, "np", doc[np.end-1].lemma_))
	db.conn.commit()
	return id 

if __name__ == "__main__":  
	print(add_text(), flush=True)
	#uvicorn.run(app, host='0.0.0.0', port=80)


mwe_pv = {
"a case of",
"a class of",
"a component of",
"a derivative of",
"a family of",
"a form of",
"a fragment of",
"a high percentage of",
"a kind of",
"a member of",
"a part of",
"a percentage of",
"a piece of",
"a portion of",
"a product of",
"a quantitative of",
"a section of",
"a segment of",
"a series of",
"a set of",
"a small percentage of",
"a sort of",
"a type of",
"a variant of",
"a variety of",
"absent from",
"acorn-shaped",
"adjacent to",
"akin to",
"almond-shaped",
"already",
"always",
"amount of",
"amounts of",
"an element of",
"an equivalent of",
"an example of",
"an ingredient of",
"analogous to",
"annual",
"annually",
"appear in",
"appeared in",
"appears in",
"apple-shaped",
"apple-sized",
"approach",
"approached",
"approaches",
"approaching",
"approximate",
"approximately",
"are",
"arise from",
"arise out of",
"arose from",
"arose out of",
"arranged",
"arrow-shaped",
"as illustrated by",
"as opposed to",
"at first",
"at last",
"at once",
"average",
"awl-shaped",
"bag-shaped",
"ball-shaped",
"bar-shaped",
"barrel-shaped",
"basket-shaped",
"be",
"bean-shaped",
"bear-sized",
"became",
"become",
"becomes",
"becoming",
"been",
"being",
"bell-shaped",
"belong to",
"belonging to",
"belongs to",
"belt-shaped",
"boat-shaped",
"boot-shaped",
"bottle-shaped",
"bowl-shaped",
"box-shaped",
"branch into",
"branched into",
"branches into",
"brick-shaped",
"brightly-colored",
"buff-colored",
"bulb-shaped",
"butterfly-shaped",
"button-shaped",
"came from",
"categories",
"categorized",
"categorizing",
"category",
"cause",
"caused",
"causes",
"change into",
"changed into",
"changes into",
"characteristic of",
"characteristics of",
"characterized by",
"cherry-sized",
"chestnut-colored",
"chocolate-colored",
"cigar-shaped",
"cinnamon-colored",
"classification",
"classifications",
"claw-shaped",
"club-shaped",
"coated with",
"come from",
"comes from",
"comparable to",
"component",
"components",
"composed of",
"comprise",
"comprised of",
"cone-shaped",
"conflict",
"conflicted",
"conflicting",
"conflicts",
"considered as",
"consist of",
"consistent of",
"consisting of",
"consists of",
"constancy",
"constant",
"constitute",
"constituted",
"constitutes",
"constituting",
"constitution",
"contain",
"contained",
"contained in",
"contained with",
"containing",
"contains",
"continual",
"continually",
"continuous",
"continuously",
"contradictory",
"converted to",
"cookie-sized",
"correspond to",
"corresponded to",
"corresponding to",
"corresponds to",
"counterpart of",
"cover",
"coverage",
"coverages",
"covered",
"covered with",
"covering",
"covers",
"cowl-shaped",
"cream-colored",
"creamy-colored",
"crescent-shaped",
"cross-shaped",
"crow-sized",
"cube-shaped",
"cucumber-shaped",
"current",
"daily",
"dark-colored",
"derive from",
"derived from",
"derives from",
"deriving from",
"develop",
"develop from",
"developed",
"developed from",
"developing",
"developing from",
"develops",
"deviod of",
"differ from",
"differed from",
"different from",
"differing from",
"differs from",
"dimension",
"disc-shaped",
"dish-shaped",
"disk-shaped",
"dissimilar",
"distinct from",
"distinguished from",
"distributed",
"divided into",
"dog-sized",
"dome-shaped",
"doughnut-shaped",
"drum-shaped",
"due to",
"dun-colored",
"duration",
"dusky-colored",
"ear-shaped",
"early",
"eel-shaped",
"egg-shaped",
"embedded in",
"embodied",
"embodies",
"embody",
"embodying",
"encompass",
"encompassed",
"encompasses",
"encompassing",
"equal to",
"equidistant",
"equipped with",
"equivalent to",
"eternal",
"ever",
"evolve into",
"evolved into",
"evolves into",
"excluded from",
"exemplified by",
"exist in",
"existed in",
"exists in",
"extend",
"extended",
"extending",
"extends",
"face",
"faced",
"faces",
"facing",
"fall into",
"falling into",
"family",
"fan-shaped",
"fawn-colored",
"feature of",
"features of",
"fell into",
"ferret-sized",
"fiddle-shaped",
"fifth",
"fig-shaped",
"filled with",
"final",
"finally",
"first class",
"first of all",
"flame-colored",
"flesh-colored",
"follow",
"followed",
"following",
"follows",
"foot-shaped",
"football-shaped",
"form",
"formation",
"formed",
"formed by",
"former",
"forms",
"found in",
"fourth",
"free of",
"frequency",
"frequent",
"funnel-shaped",
"future",
"garnet-colored",
"gave away to",
"gave rise to",
"give away to",
"give rise to",
"gives away to",
"gives rise to",
"goblet-shaped",
"gold-colored",
"gradual",
"gradually",
"grape-sized",
"grew",
"group",
"groups",
"grow",
"growing",
"grows",
"guitar-shaped",
"h-shaped",
"hammer-shaped",
"hardly",
"harp-shaped",
"heart-shaped",
"helmet-shaped",
"henceforth",
"henceforward",
"hitherto",
"honey-colored",
"hook-shaped",
"horn-shaped",
"hourglass-shaped",
"human-sized",
"identical",
"identical with",
"ill-formed",
"immediate",
"immediately",
"in shades of",
"in shape",
"in the end",
"in the form of",
"in the shape of",
"include",
"included",
"includes",
"including",
"indistinguishable from",
"inserted into",
"interpreted as",
"is",
"keel-shaped",
"kidney-shaped",
"king-sized",
"kiwi-sized",
"l-shaped",
"lack",
"lacks",
"lain",
"lance-shaped",
"lancet-shaped",
"last",
"lasted",
"lasting",
"lasts",
"late",
"later",
"lay",
"lead-colored",
"lie",
"life-sized",
"light-colored",
"like",
"lilac-colored",
"lip-shaped",
"liver-colored",
"located",
"look",
"look like",
"looked like",
"looks like",
"lying",
"lyre-shaped",
"made from",
"made up of",
"maintain",
"maintained",
"maintains",
"man-sized",
"match",
"matched",
"matches",
"matching",
"measure",
"measured",
"measures",
"measuring",
"medium-sized",
"member",
"members",
"membership",
"metal-colored",
"metallic-colored",
"moderate-sized",
"monthly",
"mouse-colored",
"mouse-sized",
"multi-colored",
"needle-shaped",
"neutral-colored",
"never",
"next",
"no longer",
"no more",
"not any longer",
"not any more",
"number of",
"numbers of",
"nutmeg-shaped",
"occur as",
"occurred as",
"occurring as",
"occurs as",
"often",
"olive-colored",
"olive-sized",
"once",
"opposite",
"orange-colored",
"orange-sized",
"originate from",
"originated from",
"originates from",
"originating from",
"oval-shaped",
"oven-shaped",
"paddle-shaped",
"pale-colored",
"palm-shaped",
"parallel to",
"parallel with",
"part of",
"partitioned into",
"parts of",
"past",
"pastel-colored",
"peach-colored",
"peachy-colored",
"pear-shaped",
"percent",
"permanent",
"pig-sized",
"pillar-shaped",
"pint-sized",
"pinwheel-shaped",
"pitcher-shaped",
"placed",
"pocket-sized",
"polar",
"portion",
"positioned at",
"possess",
"possessed",
"possesses",
"possession",
"post-communist",
"post-free",
"post-haste",
"post-hole_digger",
"post-horse",
"post-impressionist",
"post-it",
"post-maturity",
"post-menopause",
"post-mortem",
"post-mortem_examinatio",
"post-obit_bond",
"post-office_box",
"post-office_box_number",
"post-paid",
"post-rotational_nystag",
"postgraduate",
"postpone",
"postwar",
"pouch-shaped",
"pre-christian",
"pre-columbian",
"pre-eclampsia",
"pre-eminently",
"pre-empt",
"pre-emption",
"pre-emptive",
"pre-emptive_strike",
"pre-emptor",
"pre-existent",
"pre-existing",
"pre-jurassic",
"pre-raphaelite",
"pre-socratic",
"precede",
"preface",
"prerequisite",
"present",
"previous",
"process of",
"produce",
"produced",
"produced by",
"produced from",
"produces",
"proportion of",
"proportional to",
"prototype of",
"provided with",
"pumpkin-shaped",
"queen-sized",
"rabbit-sized",
"range from",
"ranged from",
"ranges from",
"ranging from",
"rate of",
"re-formed",
"regarded as",
"regular",
"regularity",
"regularly",
"render",
"rendered",
"rendering",
"renders",
"represent",
"representative",
"representative as",
"represented",
"represented by",
"representing",
"represents",
"resemblance",
"resemble",
"reside",
"resided",
"resides",
"result from",
"result in",
"resulted from",
"resulted in",
"results from",
"results in",
"ribbon-shaped",
"ring-shaped",
"rod-shaped",
"rose-colored",
"rosy-colored",
"rust-colored",
"s-shaped",
"saddle-shaped",
"sat",
"saucer-shaped",
"sausage-shaped",
"second rate",
"seldom",
"self-colored",
"separated",
"shape",
"share",
"shared",
"shares",
"sharing",
"shield-shaped",
"shift",
"shifts",
"shrew-sized",
"sickle-shaped",
"silver-colored",
"similar to",
"simultaneous",
"simultaneously",
"sit",
"sits",
"sitting",
"situate",
"situated",
"situates",
"situating",
"sixth",
"slipper-shaped",
"solid-colored",
"sometimes",
"soon",
"spade-shaped",
"sparrow-sized",
"spatula-shaped",
"spearhead-shaped",
"spider-shaped",
"spindle-shaped",
"spoon-shaped",
"square-shaped",
"squirrel-sized",
"stand",
"standing",
"stands",
"star-shaped",
"stirrup-shaped",
"stood",
"straw-colored",
"subdivided into",
"subunit",
"sudden",
"suddenly",
"surround",
"surrounded by",
"switch",
"switched",
"switches",
"sword-shaped",
"synonymous with",
"system",
"systematic",
"t-shaped",
"tadpole-shaped",
"tail-shaped",
"tassel-shaped",
"tawny-colored",
"temporal",
"the feature of",
"the features of",
"the finding of",
"the reason why",
"the remaining",
"the result of",
"the same as",
"there",
"therefore",
"thereupon",
"thimble-shaped",
"third",
"thought of as",
"timely",
"to the exclusion of",
"tongue-shaped",
"trait of",
"traits of",
"transform into",
"transformed into",
"transforming into",
"transforms into",
"tree-shaped",
"trumpet-shaped",
"tube-shaped",
"turkey-sized",
"turn into",
"turned into",
"turnip-shaped",
"turns into",
"twice",
"two-part",
"u-shaped",
"umbrella-shaped",
"unit",
"united",
"units",
"unlike",
"urgency",
"urgent",
"urn-shaped",
"v-shaped",
"varied from",
"varies from",
"varieties",
"variety",
"vary from",
"varying from",
"vase-shaped",
"violet-colored",
"volume of",
"volumes of",
"w-shaped",
"was",
"watermelon-shaped",
"wedge-shaped",
"weekly",
"weigh",
"weighed",
"weighs",
"well-formed",
"were",
"wine-colored",
"wing-shaped",
"with frequency",
"wolf-sized",
"worm-shaped",
"y-shaped",
"yearly",
"yet",
"yield",
"yielded",
"yields",
}

if __name__ == '__main__':
	import redis
	r = redis.Redis(host='dev.werror.com', port=4328)
	for w in mwe_pv: 
		r.sadd("__dic:mwe_pv", w )
	print(">> finished loading mwe_pv")


#pip install -U sentence-transformers
import fire ,json
from sentence_transformers import SentenceTransformer
model = SentenceTransformer('all-MiniLM-L6-v2')

def readline(infile, sepa=None):
	with open(infile, 'r') as fp:
		while True:
			line = fp.readline()
			if not line: break
			yield line.strip().split(sepa) if sepa else line.strip()

class util(object):
	def __init__(self): pass 

	def vec(self, infile, outfile=None): 
		''' gzjc.snt -> gzjc.vec_source  '''
		if not outfile : outfile = infile.split('.')[0] + ".vec"
		print ("started:", infile, flush=True)
		with open(outfile, 'w') as fw: 
			for line in readline(infile): 
				vec = model.encode(line.strip())
				arr = {'snt': line.strip(), 'vec': vec.tolist()}
				fw.write(json.dumps(arr) + "\n")
		print ("finished:", infile)

if __name__ == '__main__':
	fire.Fire(util)

'''
ubuntu@VM-64-245-ubuntu:~/cikuu/pypi/sbert$ nohup python __main__.py vec_source dic.snt & 
[1] 364344

	def vec(self, infile, outfile=None): 
		if not outfile : outfile = infile + ".vec"
		print ("started:", infile, flush=True)
		with open(outfile, 'w') as fw: 
			for line in readline(infile): 
				vec = model.encode(line.strip())
				fw.write(f"{line.strip()}\t{json.dumps(vec.tolist())}\n")
		print ("finished:", infile)

'''